@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: options.h,v 1.1 2004/05/23 21:23:59 hanke Exp $
+ * $Id: options.h,v 1.7 2006/01/08 13:36:56 hanke Exp $
  */
 
 #include <getopt.h>
@@ -31,8 +31,14 @@ char *output_module;
 char *language;
 char *voice_type;
 char *punctuation_mode;
+char *priority;
 int spelling;
+int wait_till_end;
+int stop_previous;
+int cancel_previous;
 
+char *application_name;
+char *connection_name;
 
 static struct option long_options[] = {
     {"rate", 1, 0, 'r'},
@@ -43,9 +49,19 @@ static struct option long_options[] = {
     {"voice-type", 1, 0, 't'},
     {"punctuation-mode", 1, 0, 'm'},
     {"spelling", 0, 0, 's'},
+    {"priority", 1, 0, 'P'},
+    {"application-name", 1, 0, 'N'},
+    {"connection-name", 1, 0, 'n'},
+    {"wait", 0, 0, 'w'},
+    {"stop", 1, 0, 'S'},
+    {"cancel", 1, 0, 'C'},
     {"version", 0, 0, 'v'},
     {"help", 0, 0, 'h'},
     {0, 0, 0, 0}
 };
 
-static char* short_options = "r:p:i:o:l:t:m:svh";
+static char* short_options = "r:p:i:l:o:t:m:sP:N:n:wSCvh";
+
+int options_parse(int argc, char *argv[]);
+void options_print_version();
+void options_print_help(char *argv[]);
