@@ -19,39 +19,43 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: festival.c,v 1.54 2004/07/22 12:59:44 hanke Exp $
+ * $Id: festival.c,v 1.69 2006/01/08 13:36:57 hanke Exp $
  */
 
 #include "fdset.h"
 #include "fdsetconv.h"
 
+#include "spd_audio.h"
+
 #include "module_utils.h"
-#include "module_utils_audio.c"
 
 #include "festival_client.c"
 
 #define MODULE_NAME     "festival"
-#define MODULE_VERSION  "0.2"
+#define MODULE_VERSION  "0.5"
 
 DECLARE_DEBUG();
 
 /* Thread and process control */
 static pthread_t festival_speak_thread;
-static pid_t festival_child_pid = 0;
 static sem_t *festival_semaphore;
 static int festival_speaking = 0;
 static int festival_pause_requested = 0;
 
 static char **festival_message;
 static EMessageType festival_message_type;
-static int festival_position = 0;
 signed int festival_volume = 0;
 
+int festival_stop_request = 0;
 int festival_stop = 0;
 
 int festival_process_pid = 0;
 
-FT_Info *festival_info;
+FT_Info *festival_info = NULL;
+
+AudioID *festival_audio_id = NULL;
+AudioOutputType festival_audio_output_method;
+char *festival_pars[10];
 
 enum{
     FCT_SOCKET = 0,
@@ -70,15 +74,8 @@ struct{
 /* Internal functions prototypes */
 void* _festival_speak(void*);
 
-size_t _festival_parent(TModuleDoublePipe dpipe, const char* message,
-                        EMessageType msgtype,
-                        const size_t maxlen, const char* dividers,
-                        int *pause_requested);
-
 void festival_parent_clean();
 
-cst_wave* festival_conv(FT_Wave *fwave);
-
 void festival_set_rate(signed int rate);
 void festival_set_pitch(signed int pitch);
 void festival_set_voice(EVoiceType voice);
@@ -90,6 +87,8 @@ void festival_set_volume(signed int volume);
 int init_festival_standalone();
 int init_festival_socket();
 
+int is_text(EMessageType msg_type);
+
 MOD_OPTION_1_INT(FestivalComunicationType);
 
 MOD_OPTION_1_INT(FestivalMaxChunkLength);
@@ -101,6 +100,11 @@ MOD_OPTION_1_INT(FestivalPitchDeviation);
 MOD_OPTION_1_INT(FestivalDebugSaveOutput);
 MOD_OPTION_1_STR(FestivalRecodeFallback);
 
+MOD_OPTION_1_STR(FestivalAudioOutputMethod);
+MOD_OPTION_1_STR(FestivalOSSDevice);
+MOD_OPTION_1_STR(FestivalALSADevice);
+MOD_OPTION_1_STR(FestivalNASServer);
+
 MOD_OPTION_1_INT(FestivalCacheOn);
 MOD_OPTION_1_INT(FestivalCacheMaxKBytes);
 MOD_OPTION_1_INT(FestivalCacheDistinguishVoices);
@@ -135,6 +139,8 @@ int cache_reset();
 int cache_insert(char* key, EMessageType msgtype, FT_Wave *value);
 FT_Wave* cache_lookup(const char *key, EMessageType msgtype, int add_counter);
 
+pthread_mutex_t sound_output_mutex;
+
 /* Public functions */
 
 int
@@ -154,70 +160,106 @@ module_load(void)
 
     MOD_OPTION_1_STR_REG(FestivalRecodeFallback, "?");
 
+    MOD_OPTION_1_STR_REG(FestivalAudioOutputMethod, "oss");
+    MOD_OPTION_1_STR_REG(FestivalOSSDevice, "/dev/dsp");
+    MOD_OPTION_1_STR_REG(FestivalALSADevice, "default");
+    MOD_OPTION_1_STR_REG(FestivalNASServer, NULL);
+
     MOD_OPTION_1_INT_REG(FestivalCacheOn, 1);
     MOD_OPTION_1_INT_REG(FestivalCacheMaxKBytes, 5120);
     MOD_OPTION_1_INT_REG(FestivalCacheDistinguishVoices, 0);
     MOD_OPTION_1_INT_REG(FestivalCacheDistinguishRate, 0);
     MOD_OPTION_1_INT_REG(FestivalCacheDistinguishPitch, 0);
 
-    /* TODO: Switch this option to 1 when the bug with the 40ms delay
+    /* TODO: Maybe switch this option to 1 when the bug with the 40ms delay
      in Festival is fixed */
     MOD_OPTION_1_INT_REG(FestivalReopenSocket, 0);
 
+    festival_audio_id = NULL;
+
     return 0;
 }
 
+#define ABORT(msg) g_string_append(info, msg); \
+	*status_info = info->str; \
+	g_string_free(info, 0); \
+	return -1;
+
 int
 module_init(char **status_info)
 {
     int ret;
+    char *error;
 
     GString *info;
 
     info = g_string_new("");
 
-    INIT_DEBUG();
-
     DBG("module_init()");
 
+    INIT_INDEX_MARKING();
+
     /* Initialize appropriate communication mechanism */
     FestivalComType = FestivalComunicationType;
     if (COM_SOCKET){
 	g_string_append(info, "Communicating with Festival through a socket. ");
 	ret = init_festival_socket();
 	if (ret == -1){
-	    g_string_append(info, "Can't connect to Festival server. Check your configuration "
-			    "in etc/speechd-modules/festival.conf for the specified host and port "
-			    "and check if Festival is really running there, e.g. with telnet. "
-			    "Please see documentation for more info.");
-	    *status_info = info->str;
-	    g_string_free(info, 0);
-	    return -1;
+	    ABORT("Can't connect to Festival server. Check your configuration "
+		  "in etc/speechd-modules/festival.conf for the specified host and port "
+		  "and check if Festival is really running there, e.g. with telnet. "
+		  "Please see documentation for more info.");
 	}else if (ret == -2){
-	    g_string_append(info, "Connect to the Festival server was successful, "
-			    "but I got disconnected immediately. This is most likely "
-			    "because of authorization problems. Check the variable "
-                            "server_access_list in etc/festival.scm and consult documentation "
-			    "for more information");
-	    *status_info = info->str;
-	    g_string_free(info, 0);
-	    return -1;
+	    ABORT("Connect to the Festival server was successful, "
+		  "but I got disconnected immediately. This is most likely "
+		  "because of authorization problems. Check the variable "
+		  "server_access_list in etc/festival.scm and consult documentation "
+		  "for more information.");
 	}
-
     }
     if (COM_LOCAL){
 	g_string_append(info, "Communicating with Festival through a local child process.");
 	if(init_festival_standalone()){
-	    g_string_append(info, "Local connect to Festival failed for unknown reasons.");
-	    *status_info = info->str;
-	    g_string_free(info, 0);
-	    return -1;
-	    
+	    ABORT("Local connect to Festival failed for unknown reasons.");
 	}
     }
 
+    DBG("Openning audio output system");
+    if (!strcmp(FestivalAudioOutputMethod, "oss")){
+	DBG("Using OSS audio output method");
+	festival_pars[0] = FestivalOSSDevice;
+	festival_pars[1] = NULL;
+	festival_audio_id = spd_audio_open(AUDIO_OSS, (void**) festival_pars, &error);
+	festival_audio_output_method = AUDIO_OSS;
+    }
+    else if (!strcmp(FestivalAudioOutputMethod, "alsa")){
+	DBG("Using Alsa audio output method");
+	festival_pars[0] = FestivalALSADevice;
+	festival_pars[1] = NULL;
+	festival_audio_id = spd_audio_open(AUDIO_ALSA, (void**) festival_pars, &error);
+	festival_audio_output_method = AUDIO_ALSA;
+    }
+    else if (!strcmp(FestivalAudioOutputMethod, "nas")){
+	DBG("Using NAS audio output method");
+	festival_pars[0] = FestivalNASServer;
+	festival_pars[1] = NULL;
+	festival_audio_id = spd_audio_open(AUDIO_NAS, (void**) festival_pars, &error);
+	festival_audio_output_method = AUDIO_NAS;
+    }
+    else{
+	ABORT("Sound output method specified in configuration not supported. "
+	      "Please choose 'oss', 'alsa' or 'nas'.");
+    }
+    if (festival_audio_id == NULL){
+	g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
+	ABORT("Can't open sound device.");
+    }
+
+    pthread_mutex_init(&sound_output_mutex, NULL);
+
     /* Initialize global variables */
     festival_message = (char**) xmalloc (sizeof (char*));    
+    *festival_message = NULL;
 
     /* Initialize festival_speak thread to handle communication
      with festival in a separate thread (to be faster in communication
@@ -242,21 +284,23 @@ module_init(char **status_info)
     g_string_free(info, 0);
     return 0;
 }
+#undef ABORT
 
 int
 module_speak(char *data, size_t bytes, EMessageType msgtype)
 {
+    int ret;
 
     DBG("module_speak()\n");
 
     if (data == NULL) return -1;
 
     if (festival_speaking){
-        DBG("Speaking when requested to write\n");
-        return -1;
+	DBG("Speaking when requested to write\n");
+	return -1;
     }
 
-    festival_stop = 0;
+    festival_stop_request = 0;
 
     /* Check the parameters */
     if(module_write_data_ok(data) != 0) return -1;
@@ -265,6 +309,7 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
     if ((msgtype == MSGTYPE_TEXT) && (msg_settings.spelling_mode == SPELLING_ON))
         festival_message_type = MSGTYPE_SPELL;
 
+
     /* If the connection crashed or language or voice
        change, we will need to set all the parameters again */
     if (COM_SOCKET){
@@ -272,8 +317,13 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
 	    DBG("Recovering after a connection loss");
 	    CLEAN_OLD_SETTINGS_TABLE();
 	    festival_info = festivalOpen(festival_info);
-	    FestivalSetMultiMode(festival_info, "t");
-	    festival_connection_crashed = 0;
+	    if (festival_info) festival_connection_crashed = 0;
+	    else{
+		DBG("Can't recover. Not possible to open connection to Festival.");
+		return -1;
+	    }
+	    ret = FestivalSetMultiMode(festival_info, "t");
+	    if (ret != 0) return -1;
 	}
     }
 
@@ -284,7 +334,6 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
         CLEAN_OLD_SETTINGS_TABLE();
     }
 
-
     /* Setting voice parameters */
     DBG("Updating parameters");
     UPDATE_STRING_PARAMETER(language, festival_set_language);
@@ -295,8 +344,14 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
     UPDATE_PARAMETER(punctuation_mode, festival_set_punctuation_mode);
     UPDATE_PARAMETER(cap_let_recogn, festival_set_cap_let_recogn);
 
+    if (festival_connection_crashed){
+	DBG("ERROR: Festival connection not working!");
+	return -1;
+    }
+
     DBG("Requested data: |%s| \n", data);
 
+    xfree(*festival_message);
     *festival_message = strdup(data);
     if (*festival_message == NULL){
         DBG("Error: Copying data unsuccesful.");
@@ -334,13 +389,16 @@ module_stop(void)
 	    //	    festival_stop_local();
 	}
 
-	if (festival_child_pid != 0){
-	    DBG("festival stopped by killing child process pid %d\n", festival_child_pid);
-	    kill(festival_child_pid, SIGKILL);
-	    festival_child_pid = 0;
+	if (!festival_stop){
+	    pthread_mutex_lock(&sound_output_mutex);       
+	    festival_stop = 1;
+	    if (festival_speaking && festival_audio_id){
+		spd_audio_stop(festival_audio_id);
+	    }
+	    pthread_mutex_unlock(&sound_output_mutex);
 	}
     }
-
+    
     return 0;
 }
 
@@ -351,46 +409,41 @@ module_pause(void)
     if(festival_speaking){
         DBG("Sending request for pause to child\n");
         festival_pause_requested = 1;
-        DBG("Waiting in pause for child to terminate\n");
-        while(festival_speaking) usleep(10000);       
-        DBG("Paused at mark: %d", current_index_mark);
-	festival_pause_requested = 0;
-        return current_index_mark;
+        DBG("Signalled to pause");
+        return 0;
     }else{
         return -1;
     }
 }
 
-int
-module_is_speaking(void)
-{
-    return festival_speaking; 
-}
-
 void
 module_close(int status)
 {
     
     DBG("festival: close()\n");
 
+    DBG("Stopping the module");
     while(festival_speaking){
         module_stop();
-        usleep(5);
+        usleep(50);
     }
 
     // DBG("festivalClose()");
     // festivalClose(festival_info);
     
+    DBG("Terminating threads");
     module_terminate_thread(festival_speak_thread);
    
-    delete_FT_Info(festival_info);
+    if (festival_info)
+	delete_FT_Info(festival_info);
 
     /* TODO: Solve this */
-    DBG("Removing junk files in tmp/");
-    system("rm -f /tmp/est* 2> /dev/null");
+    //    DBG("Removing junk files in tmp/");
+    //    system("rm -f /tmp/est* 2> /dev/null");
 
-    DBG("Closing debug file");
-    CLOSE_DEBUG_FILE();
+    DBG("Closing audio output");
+    if (festival_audio_id)
+	spd_audio_close(festival_audio_id);
 
     exit(status);
 }
@@ -398,35 +451,245 @@ module_close(int status)
 
 /* Internal functions */
 
+#define CLEAN_UP(code, im) \
+        { \
+          if(!wave_cached) if (fwave) delete_FT_Wave(fwave); \
+          pthread_mutex_lock(&sound_output_mutex); \
+          festival_stop = 0; \
+          festival_speaking = 0; \
+          pthread_mutex_unlock(&sound_output_mutex); \
+          im(); \
+          goto sem_wait; \
+        }
+
+#define CLP(code, im) \
+        { \
+          pthread_mutex_lock(&sound_output_mutex); \
+          festival_stop = 0; \
+          festival_speaking = 0; \
+          pthread_mutex_unlock(&sound_output_mutex); \
+          im(); \
+          goto sem_wait; \
+        }
+
+
+int
+festival_send_to_audio(FT_Wave *fwave)
+{
+    AudioTrack track;
+    int ret;
+    
+    track.num_samples = fwave->num_samples;
+    track.num_channels = 1;
+    track.sample_rate = fwave->sample_rate;
+    track.bits = 16;
+    track.samples = fwave->samples;
+    
+    if (track.samples != NULL){
+	DBG("Sending to audio");
+	ret = spd_audio_play(festival_audio_id, track);
+	if (ret < 0) DBG("ERROR: Can't play track for unknown reason.");
+	DBG("Sent to audio.");
+    }
+
+    return 0;
+}
 
 void*
 _festival_speak(void* nothing)
 {	
 
+    int ret;
+    int bytes;
+    int wave_cached;
+    FT_Wave *fwave;
+    int debug_count = 0;
+
+    int terminate = 0;
+    int r;
+
+    char *callback;
+
     DBG("festival: speaking thread starting.......\n");
 
     cache_init();
 
-    module_speak_thread_wfork(festival_semaphore, &festival_child_pid,
-                              module_audio_output_child, _festival_parent, 
-                              &festival_speaking, festival_message, &festival_message_type,
-                              -1, NULL,
-                              &festival_position, &festival_pause_requested);
+    set_speaking_thread_parameters();
+
+    while(1){        
+    sem_wait:
+        sem_wait(festival_semaphore);
+        DBG("Semaphore on, speaking\n");
+
+	spd_audio_set_volume(festival_audio_id, festival_volume);
+
+	festival_stop = 0;	
+	festival_speaking = 1;
+	wave_cached = 0;
+	fwave = NULL;
+
+	terminate = 0;     
+
+	bytes = strlen(*festival_message);
+
+	module_report_event_begin();
+
+	DBG("Going to synthesize: |%s|", *festival_message);
+	if (bytes > 0){
+	    if (!is_text(festival_message_type)){	/* it is a raw text */	     
+		DBG("Cache mechanisms...");
+		fwave = cache_lookup(*festival_message, festival_message_type, 1);
+		if (fwave != NULL){
+		    wave_cached = 1;
+		    if (fwave->num_samples != 0){
+			if(FestivalDebugSaveOutput){
+			    char filename_debug[256];
+			    sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
+			    save_FT_Wave_snd(fwave, filename_debug);
+			}
+						
+			festival_send_to_audio(fwave);
+			
+			DBG("Here");
+
+			if (!festival_stop){
+			    DBG("Here1");
+			
+			    CLEAN_UP(0, module_report_event_end);
+			}else{
+			    DBG("Here2");
+			
+			    CLEAN_UP(0, module_report_event_stop);
+			}
+
+		    }else{
+			CLEAN_UP(0, module_report_event_end);
+		    }
+		}
+	    }
+	    
+	    /*  Set multi-mode for appropriate kind of events */
+	    if (is_text(festival_message_type)){	/* it is a raw text */
+		ret = FestivalSetMultiMode(festival_info, "t");
+		if (ret != 0) CLP(0, module_report_event_stop);
+	    }else{			/* it is some kind of event */
+		ret = FestivalSetMultiMode(festival_info, "nil");
+		if (ret != 0) CLP(0, module_report_event_stop);
+	    }
+
+	    switch(festival_message_type)
+		{
+		case MSGTYPE_TEXT: r = festivalStringToWaveRequest(festival_info, *festival_message); break;
+		case MSGTYPE_SOUND_ICON: r = festivalSoundIcon(festival_info, *festival_message); break;
+		case MSGTYPE_CHAR: r = festivalCharacter(festival_info, *festival_message); break;
+		case MSGTYPE_KEY: r = festivalKey(festival_info, *festival_message); break;
+		case MSGTYPE_SPELL: r = festivalSpell(festival_info, *festival_message); break;
+		}
+	    if (r < 0){
+		DBG("Couldn't process the request to say the object.");
+		CLP(0, module_report_event_stop);
+	    }
+	}
+    
+	while(1){
+
+	    wave_cached = 0;
+	    DBG("Retrieving data\n");	   
+
+	    /* (speechd-next) */
+	    if (is_text(festival_message_type)){
+
+		if (festival_stop){
+		    DBG("Module stopped 1");
+		    CLEAN_UP(0, module_report_event_stop);
+		}
+
+		DBG("Getting data in multi mode");
+		fwave = festivalGetDataMulti(festival_info, &callback,
+					     &festival_stop_request,
+					     FestivalReopenSocket);
+
+		if (callback != NULL){ 
+		    if((festival_pause_requested) && (!strncmp(callback, INDEX_MARK_BODY,
+							       INDEX_MARK_BODY_LEN))){
+			DBG("Pause requested, pausing.");
+			module_report_index_mark(callback);
+			xfree(callback);
+			festival_pause_requested = 0;
+			CLEAN_UP(0, module_report_event_pause);
+		    }else{
+			module_report_index_mark(callback);
+			xfree(callback);
+			continue;
+		    }
+		}
+	    }else{			/* is event */
+		DBG("Getting data in single mode");
+		fwave = festivalStringToWaveGetData(festival_info);
+		terminate = 1;
+		callback = NULL;
+	    }
+
+	    if (fwave == NULL){
+		DBG("End of sound samples, terminating this message...");
+		CLEAN_UP(0, module_report_event_end);
+	    }
+	
+	    if (festival_message_type == MSGTYPE_CHAR 
+		|| festival_message_type == MSGTYPE_KEY 
+		|| festival_message_type == MSGTYPE_SOUND_ICON){
+		DBG("Storing record for %s in cache\n", *festival_message);
+		/* cache_insert takes care of not inserting the same
+		 message again */
+		cache_insert(strdup(*festival_message), festival_message_type, fwave);
+		wave_cached = 1;
+	    }
+
+	    if (festival_stop){
+		DBG("Module stopped 2");
+		CLEAN_UP(0, module_report_event_stop);
+	    }       	
+
+	    if (fwave->num_samples != 0){
+		DBG("Sending message to audio: %d bytes\n",
+		    (fwave->num_samples) * sizeof(short));	
+    
+		if(FestivalDebugSaveOutput){
+		    char filename_debug[256];
+		    sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
+		    save_FT_Wave_snd(fwave, filename_debug);
+		}
+	    
+		DBG("Playing sound samples");
+		festival_send_to_audio(fwave);
+
+		if (!wave_cached) delete_FT_Wave(fwave);
+		DBG("End of playing sound samples");
+	    }            
+
+	    if (terminate){
+		DBG("Ok, end of samples, returning");
+		CLP(0, module_report_event_end);
+	    }
+
+	    if (festival_stop){
+		DBG("Module stopped 3");
+		CLP(0, module_report_event_stop);
+	    }
+	}
+
+    }
+
+
+    festival_stop = 0;
     festival_speaking = 0;
-
+	
     DBG("festival: speaking thread ended.......\n");    
-
+    
     pthread_exit(NULL);
 }	
 
 
-#define CLEAN_UP(code) \
-        { \
-          if(!wave_cached) delete_FT_Wave(fwave); \
-          module_parent_dp_close(dpipe); \
-          return(code); \
-        }
-
 int
 is_text(EMessageType msg_type)
 {
@@ -435,154 +698,6 @@ is_text(EMessageType msg_type)
 }
 
 
-size_t
-_festival_parent(TModuleDoublePipe dpipe, const char* message,
-		 EMessageType msgtype, const size_t maxlen, const char* dividers,
-		 int *pause_requested)
-{
-    int ret;
-    int bytes;
-    FT_Wave *fwave = NULL;
-    int terminate = 0;
-    int r = -1;
-    static int debug_count = 0;
-    int l;
-    int h;
-    char *tptr;
-    int wave_cached = 0;
-
-    char *callback;
-
-    DBG("Entering parent process, closing pipes");
-
-    bytes = strlen(message);
-
-    module_parent_dp_init(dpipe);
-    
-    DBG("Synthesizing: |%s|", message);
-    if (bytes > 0){
-	if (is_text(msgtype)){	/* is it a raw text */
-	    FestivalSetMultiMode(festival_info, "t");
-	}else{			/* it is some kind of event */
-	    FestivalSetMultiMode(festival_info, "nil");
-	    DBG("Cache mechanisms...");
-	    fwave = cache_lookup(message, msgtype, 1);
-	    if (fwave != NULL){
-		wave_cached = 1;
-		if (fwave->num_samples != 0){
-		    if(FestivalDebugSaveOutput){
-			char filename_debug[256];
-			sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
-			save_FT_Wave_snd(fwave, filename_debug);
-		    }
-		    ret = module_parent_send_samples(dpipe, fwave->samples,
-						     fwave->num_samples, fwave->sample_rate,
-							 festival_volume);
-		    if (ret == -1) CLEAN_UP(-1);
-		    
-		    DBG("parent (1): Sent %d bytes\n", ret);            
-		    ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
-		    if (ret == -1) CLEAN_UP(-1);
-		    
-		    ret = module_parent_wait_continue(dpipe);
-		    if (ret != 0) CLEAN_UP(-1);
-		    
-		    DBG("End of data in parent (1), closing pipes");
-		    
-		    CLEAN_UP(0);
-		}else
-		    CLEAN_UP(0);
-	    }
-	}
-	
-	switch(msgtype)
-	    {
-	    case MSGTYPE_TEXT: r = festivalStringToWaveRequest(festival_info, message); break;
-	    case MSGTYPE_SOUND_ICON: r = festivalSoundIcon(festival_info, message); break;
-	    case MSGTYPE_CHAR: r = festivalCharacter(festival_info, message); break;
-	    case MSGTYPE_KEY: r = festivalKey(festival_info, message); break;
-	    case MSGTYPE_SPELL: r = festivalSpell(festival_info, message); break;
-	    }
-	if (r < 0){
-	    DBG("Couldn't process the request to say the object.");
-	    CLEAN_UP(-1);
-	}
-    }
-    
-
-    while(1){
-
-	DBG("Retrieving data\n");
-
-	/* (speechd-next) */
-	if (is_text(msgtype)){
-	    DBG("Getting data in multi mode");
-	    fwave = festivalGetDataMulti(festival_info, &callback, &festival_stop, FestivalReopenSocket);
-
-	    if (callback != NULL){
-		DBG("Callback detected: %s", callback);
-		l = strlen(callback);
-		if (l<4) continue;	/* some other index mark */
-		if (strncmp(callback, "sdm_", 3)) continue;	/* some other index mark */
-		h = strtol(callback+3, &tptr, 10);
-		if (tptr != callback+3) current_index_mark = h;
-		DBG("Current index mark is set to %d", current_index_mark);
-		if(*pause_requested){
-		    DBG("Pause requested, not sending more data to child");
-		    CLEAN_UP(0);
-		}else{
-		    continue;
-		}
-	    }
-	}else{			/* is event */
-	    DBG("Getting data in single mode");
-	    fwave = festivalStringToWaveGetData(festival_info);
-	    terminate = 1;
-	    callback = NULL;
-	}
-
-	if (fwave == NULL){
-	    DBG("End of sound samples, returning...");
-	    CLEAN_UP(0);
-	}
-	
-	if (msgtype == MSGTYPE_CHAR || msgtype == MSGTYPE_KEY ||
-	    msgtype == MSGTYPE_SOUND_ICON){
-	    DBG("Storing record for %s in cache\n", message);
-	    wave_cached = 1;
-	    cache_insert(strdup(message), msgtype, fwave);
-	}
-	
-	DBG("Sending message to child in wav: %d bytes\n",
-	    (fwave->num_samples) * sizeof(short));
-	
-	if (fwave->num_samples != 0){
-	    
-	    if(FestivalDebugSaveOutput){
-		char filename_debug[256];
-		sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
-		save_FT_Wave_snd(fwave, filename_debug);
-	    }
-	    
-	    ret = module_parent_send_samples(dpipe, fwave->samples,
-					     fwave->num_samples, fwave->sample_rate,
-					     festival_volume);
-	    if (ret == -1) CLEAN_UP(-1);
-	    DBG("parent: Sent %d bytes\n", ret);            
-	    
-	    ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
-	    if (ret == -1) CLEAN_UP(-1);
-	    ret = module_parent_wait_continue(dpipe);
-	    if (ret != 0) CLEAN_UP(-1);
-	}            
-
-	if (terminate){
-	    DBG("Ok, end of samples, returning");
-	    CLEAN_UP(0);
-	}
-    }
-}
-
 void
 festival_set_language(char* language)
 {   
@@ -640,29 +755,6 @@ festival_set_cap_let_recogn(ECapLetRecogn recogn)
     xfree(recogn_mode);
 }
 
-/* Warning: In order to be faster, this function doesn't copy
- the samples from cst_wave to fwave, so be sure to free the
- fwave only, not fwave->samples after conversion! */
-cst_wave*
-festival_conv(FT_Wave *fwave)
-{
-    cst_wave *ret;
-
-    if (fwave == NULL) return NULL;
-
-    ret = (cst_wave*) xmalloc(sizeof(cst_wave));
-    ret->type = strdup("riff");
-    ret->num_samples = fwave->num_samples;
-    ret->sample_rate = fwave->sample_rate;
-    ret->num_channels = 1;
-
-    /* The data isn't copied, only the pointer! */
-    ret->samples = fwave->samples;
-
-    return ret;
-}
-
-
 
 /* --- Cache related functions --- */
 
@@ -803,6 +895,8 @@ cache_gen_key(EMessageType type)
 
     if (msg_settings.language == NULL) return NULL;
 
+    DBG("v, p, r = %d %d %d", FestivalCacheDistinguishVoices, FestivalCacheDistinguishPitch, FestivalCacheDistinguishRate);
+
     if (FestivalCacheDistinguishVoices) kvoice = msg_settings.voice;
     if (FestivalCacheDistinguishPitch) kpitch = msg_settings.pitch;
     if (FestivalCacheDistinguishRate) krate = msg_settings.rate;
