@@ -19,19 +19,19 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: flite.c,v 1.42 2004/07/21 08:14:31 hanke Exp $
+ * $Id: flite.c,v 1.50 2006/01/08 13:36:57 hanke Exp $
  */
 
 
 #include <flite/flite.h>
+#include "spd_audio.h"
 
 #include "fdset.h"
 
 #include "module_utils.h"
-#include "module_utils_audio.c"
 
 #define MODULE_NAME     "flite"
-#define MODULE_VERSION  "0.1"
+#define MODULE_VERSION  "0.5"
 
 #define DEBUG_MODULE 1
 DECLARE_DEBUG();
@@ -40,7 +40,6 @@ DECLARE_DEBUG();
 static int flite_speaking = 0;
 
 static pthread_t flite_speak_thread;
-static pid_t flite_pid;
 static sem_t *flite_semaphore;
 
 static char **flite_message;
@@ -49,25 +48,34 @@ static EMessageType flite_message_type;
 static int flite_position = 0;
 static int flite_pause_requested = 0;
 
+signed int flite_volume = 0;
+
 /* Internal functions prototypes */
 static void flite_set_rate(signed int rate);
 static void flite_set_pitch(signed int pitch);
+static void flite_set_volume(signed int pitch);
 static void flite_set_voice(EVoiceType voice);
 
 static void* _flite_speak(void*);
-static void _flite_child(TModuleDoublePipe dpipe, const size_t maxlen);
-static void flite_child_close(TModuleDoublePipe dpipe);
 
 /* Voice */
 cst_voice *register_cmu_us_kal();
 cst_voice *flite_voice;
 
-/* Fill the module_info structure with pointers to this modules functions */
-//DECLARE_MODINFO("flite", "Flite software synthesizer v. 1.2");
+AudioID *flite_audio_id = NULL;
+AudioOutputType flite_audio_output_method;
+char *flite_pars[10];
+int flite_stop = 0;
+
 
 MOD_OPTION_1_INT(FliteMaxChunkLength);
 MOD_OPTION_1_STR(FliteDelimiters);
 
+MOD_OPTION_1_STR(FliteAudioOutputMethod);
+MOD_OPTION_1_STR(FliteOSSDevice);
+MOD_OPTION_1_STR(FliteNASServer);
+MOD_OPTION_1_STR(FliteALSADevice);
+
 /* Public functions */
 
 int
@@ -80,17 +88,32 @@ module_load(void)
    MOD_OPTION_1_INT_REG(FliteMaxChunkLength, 300);
    MOD_OPTION_1_STR_REG(FliteDelimiters, ".");
 
+   MOD_OPTION_1_STR_REG(FliteAudioOutputMethod, "oss");
+   MOD_OPTION_1_STR_REG(FliteOSSDevice, "/dev/dsp");
+   MOD_OPTION_1_STR_REG(FliteNASServer, NULL);
+   MOD_OPTION_1_STR_REG(FliteALSADevice, "default");
+
    return 0;
 }
 
+#define ABORT(msg) g_string_append(info, msg); \
+        DBG("FATAL ERROR:", info->str); \
+	*status_info = info->str; \
+	g_string_free(info, 0); \
+	return -1;
+
 int
 module_init(char **status_info)
 {
     int ret;
+    char *error;
+    GString *info;
+
+    DBG("Module init");
+    INIT_INDEX_MARKING();
 
     *status_info = NULL;    
-
-    INIT_DEBUG();
+    info = g_string_new("");
 
     /* Init flite and register a new voice */
     flite_init();
@@ -103,11 +126,44 @@ module_init(char **status_info)
 			      "instalation is incomplete.");
         return -1;
     }
-    
+
+    DBG("Openning audio");
+    if (!strcmp(FliteAudioOutputMethod, "oss")){
+	DBG("Using OSS sound output.");
+	flite_pars[0] = strdup(FliteOSSDevice);
+	flite_pars[1] = NULL;
+	flite_audio_id = spd_audio_open(AUDIO_OSS, (void**) flite_pars, &error);
+	flite_audio_output_method = AUDIO_OSS;
+    }
+    else if (!strcmp(FliteAudioOutputMethod, "nas")){
+	DBG("Using NAS sound output.");
+	flite_pars[0] = FliteNASServer;
+	flite_pars[1] = NULL;
+	flite_audio_id = spd_audio_open(AUDIO_NAS, (void**) flite_pars, &error);
+	flite_audio_output_method = AUDIO_NAS;
+    }
+    else if (!strcmp(FliteAudioOutputMethod, "alsa")){
+	DBG("Using NAS sound output.");
+	flite_pars[0] = FliteALSADevice;
+	flite_pars[1] = NULL;
+	flite_audio_id = spd_audio_open(AUDIO_ALSA, (void**) flite_pars, &error);
+	flite_audio_output_method = AUDIO_ALSA;
+    }
+    else{	
+	ABORT("Sound output method specified in configuration not supported. "
+	      "Please choose 'oss' or 'nas'.");
+    }
+    if (flite_audio_id == NULL){
+	g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
+	ABORT("Can't open sound device.");
+    }
+
     DBG("FliteMaxChunkLength = %d\n", FliteMaxChunkLength);
     DBG("FliteDelimiters = %s\n", FliteDelimiters);
     
-    flite_message = malloc (sizeof (char*));    
+    flite_message = malloc (sizeof (char*));
+    *flite_message = NULL;
+
     flite_semaphore = module_semaphore_init();
 
     DBG("Flite: creating new thread for flite_speak\n");
@@ -126,13 +182,11 @@ module_init(char **status_info)
 								
     return 0;
 }
-
+#undef ABORT
 
 int
 module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 {
-    char *temp;
-
     DBG("write()\n");
 
     if (flite_speaking){
@@ -142,14 +196,19 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
     
     if(module_write_data_ok(data) != 0) return -1;
 
+    DBG("Requested data: |%s|\n", data);
+
+    if (*flite_message != NULL){
+	xfree(*flite_message);
+	*flite_message = NULL;
+    }
     *flite_message = module_strip_ssml(data);
     flite_message_type = MSGTYPE_TEXT;
-
-    DBG("Requested data: |%s|\n", data);
 	
     /* Setting voice */
     UPDATE_PARAMETER(voice, flite_set_voice);
     UPDATE_PARAMETER(rate, flite_set_rate);
+    UPDATE_PARAMETER(volume, flite_set_volume);
     UPDATE_PARAMETER(pitch, flite_set_pitch);
 
     /* Send semaphore signal to the speaking thread */
@@ -163,12 +222,16 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 int
 module_stop(void) 
 {
+    int ret;
     DBG("flite: stop()\n");
 
-    if(flite_speaking && (flite_pid != 0)){
-        DBG("flite: killing process pid %d\n", flite_pid);
-        kill(flite_pid, SIGKILL);
+    flite_stop = 1;
+    if (flite_audio_id){
+	DBG("Stopping audio");
+	ret = spd_audio_stop(flite_audio_id);
+	if (ret != 0) DBG("WARNING: Non 0 value from spd_audio_stop: %d", ret);
     }
+
     return 0;
 }
 
@@ -177,43 +240,36 @@ module_pause(void)
 {
     DBG("pause requested\n");
     if(flite_speaking){
+        DBG("Flite doesn't support pause, stopping\n");
 
-        DBG("Sending request to pause to child\n");
-        flite_pause_requested = 1;
-        DBG("Waiting in pause for child to terminate\n");
-        while(flite_speaking) usleep(10);
+	module_stop();
 
-        DBG("paused at byte: %d", flite_position);
-        return flite_position;
-        
+        return -1;
     }else{
         return 0;
     }
 }
 
-int
-module_is_speaking(void)
-{
-    return flite_speaking; 
-}
-
 void
 module_close(int status)
 {
     
     DBG("flite: close()\n");
 
+    DBG("Stopping speech");
     if(flite_speaking){
         module_stop();
     }
 
+    DBG("Terminating threads");
     if (module_terminate_thread(flite_speak_thread) != 0)
         exit(1);
    
     xfree(flite_voice);
-    
-    CLOSE_DEBUG_FILE();
 
+    DBG("Closing audio output");
+    spd_audio_close(flite_audio_id);
+    
     exit(status);
 }
 
@@ -223,14 +279,121 @@ module_close(int status)
 void*
 _flite_speak(void* nothing)
 {	
+    AudioTrack track;
+    cst_wave *wav;
+    unsigned int pos;
+    char *buf;
+    int bytes;
+    int ret;
 
     DBG("flite: speaking thread starting.......\n");
 
-    module_speak_thread_wfork(flite_semaphore, &flite_pid,
-                              _flite_child, module_parent_wfork, 
-                              &flite_speaking, flite_message, &flite_message_type,
-                              FliteMaxChunkLength, FliteDelimiters,
-                              &flite_position, &flite_pause_requested);
+    set_speaking_thread_parameters();
+
+    while(1){        
+        sem_wait(flite_semaphore);
+        DBG("Semaphore on\n");
+
+	flite_stop = 0;
+	flite_speaking = 1;       
+
+	spd_audio_set_volume(flite_audio_id, flite_volume);
+
+	/* TODO: free(buf) */
+	buf = (char*) malloc((FliteMaxChunkLength+1) * sizeof(char));
+	pos = 0;
+	module_report_event_begin();
+	while(1){
+	    if (flite_stop){
+		DBG("Stop in child, terminating");
+		flite_speaking = 0;
+		module_report_event_stop();
+		break;
+	    }
+	    bytes = module_get_message_part(*flite_message, buf, &pos, 
+					    FliteMaxChunkLength, FliteDelimiters);
+
+	    if (bytes < 0){
+		DBG("ERROR: Can't get message part, terminating");
+		flite_speaking = 0;
+		module_report_event_stop();
+		break;
+	    }
+
+	    buf[bytes] = 0;
+	    DBG("Returned %d bytes from get_part\n", bytes);
+	    DBG("Text to synthesize is '%s'\n", buf);
+	    
+	    if (flite_pause_requested && (current_index_mark!=-1)){
+		DBG("Pause requested in parent, position %d\n", current_index_mark);                
+		flite_pause_requested = 0;
+		flite_position = current_index_mark;
+		break;
+	    }
+
+	    if (bytes > 0){
+		DBG("Speaking in child...");
+
+		DBG("Trying to synthesize text");
+		wav = flite_text_to_wave(buf, flite_voice);
+
+		if (wav == NULL){
+		    DBG("Stop in child, terminating");
+		    flite_speaking = 0;
+		    module_report_event_stop();
+		    break;
+		}
+
+		track.num_samples = wav->num_samples;
+		track.num_channels = wav->num_channels;
+		track.sample_rate = wav->sample_rate;
+		track.bits = 16;
+		track.samples = wav->samples;
+
+		DBG("Got %d samples", track.num_samples);
+		if (track.samples != NULL){
+		    if (flite_stop){
+			DBG("Stop in child, terminating");
+			flite_speaking = 0;
+			module_report_event_stop();
+			delete_wave(wav);
+			break;
+		    }
+		    DBG("Playing part of the message");
+		    ret = spd_audio_play(flite_audio_id, track);
+		    if (ret < 0) DBG("ERROR: spd_audio failed to play the track");
+		    if (flite_stop){
+			DBG("Stop in child, terminating (s)");
+			flite_speaking = 0;
+			module_report_event_stop();
+			delete_wave(wav);
+			break;
+		    }
+		}
+		delete_wave(wav);
+	    }
+	    else if (bytes == -1){
+		DBG("End of data in speaking thread");
+		flite_speaking = 0;
+		module_report_event_end();
+		break;
+	    }else{
+		flite_speaking = 0;
+		module_report_event_end();
+		break;
+	    }
+
+	    if (flite_stop){
+		DBG("Stop in child, terminating");
+		flite_speaking = 0;
+		module_report_event_stop();
+		break;
+	    }
+	}
+	flite_stop = 0;
+	xfree(buf);
+    }
+
     flite_speaking = 0;
 
     DBG("flite: speaking thread ended.......\n");    
@@ -238,52 +401,6 @@ _flite_speak(void* nothing)
     pthread_exit(NULL);
 }	
 
-void
-_flite_child(TModuleDoublePipe dpipe, const size_t maxlen)
-{
-    char *text;  
-    sigset_t some_signals;
-    int bytes;
-
-    sigfillset(&some_signals);
-    module_sigunblockusr(&some_signals);
-
-    module_child_dp_init(dpipe);
-
-    text = malloc((maxlen + 1) * sizeof(char));
-
-    DBG("Entering child loop\n");
-    while(1){
-        /* Read the waiting data */
-        bytes = module_child_dp_read(dpipe, text, maxlen);
-        DBG("read %d bytes in child", bytes);
-        if (bytes == 0){
-            free(text);
-            flite_child_close(dpipe);
-        }
-
-        //        text[bytes] = '.';
-        text[bytes] = 0;
-
-        DBG("child: got data |%s|, bytes:%d", text, bytes);
-
-        DBG("Speaking in child...");
-        module_sigblockusr(&some_signals);        
-        {
-	    const char outtype[] = "play";
-	    flite_text_to_speech(text, flite_voice, outtype);
-	    /*            spd_audio_open(wave);
-			  spd_audio_play_wave(wave);
-			  spd_audio_close(); */
-        }
-        module_sigunblockusr(&some_signals);        
-
-        DBG("child->parent: ok, send more data");      
-        module_child_dp_write(dpipe, "C", 1);
-    }
-}
-
-
 static void
 flite_set_rate(signed int rate)
 {
@@ -296,6 +413,13 @@ flite_set_rate(signed int rate)
                    stretch);
 }
 
+static void
+flite_set_volume(signed int volume)
+{
+    assert(volume >= -100 && volume <= +100);
+    flite_volume = volume;
+}
+
 static void
 flite_set_pitch(signed int pitch)
 {
@@ -315,14 +439,5 @@ flite_set_voice(EVoiceType voice)
     }
 }
 
-static void
-flite_child_close(TModuleDoublePipe dpipe)
-{   
-    spd_audio_close();
-    DBG("child: Pipe closed, exiting, closing pipes..\n");
-    module_child_dp_close(dpipe);          
-    DBG("Child ended...\n");
-    exit(0);
-}
 
 #include "module_main.c"
