@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: generic.c,v 1.17 2004/07/21 08:14:35 hanke Exp $
+ * $Id: generic.c,v 1.22 2005/12/07 08:31:32 hanke Exp $
  */
 
 #include <glib.h>
@@ -80,16 +80,13 @@ MOD_OPTION_1_FLOAT(GenericVolumeMultiply);
 MOD_OPTION_1_INT(GenericVolumeForceInteger);
 MOD_OPTION_3_HT(GenericLanguage, code, name, charset);
 
-
 static char generic_msg_pitch_str[16];
 static char generic_msg_rate_str[16];
 static char generic_msg_volume_str[16];
 static char* generic_msg_voice_str = NULL;
 static TGenericLanguage* generic_msg_language = NULL;
 
-
 /* Public functions */
-
 int
 module_load(void)
 {
@@ -132,8 +129,6 @@ module_init(char **status_info)
 
     *status_info = NULL;
 
-    INIT_DEBUG();
-
     DBG("GenericMaxChunkLength = %d\n", GenericMaxChunkLength);
     DBG("GenericDelimiters = %s\n", GenericDelimiters);
     DBG("GenericExecuteSynth = %s\n", GenericExecuteSynth);
@@ -180,7 +175,6 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 
     UPDATE_STRING_PARAMETER(language, generic_set_language);
     UPDATE_PARAMETER(voice, generic_set_voice);
-
     UPDATE_PARAMETER(pitch, generic_set_pitch);
     UPDATE_PARAMETER(rate, generic_set_rate);
     UPDATE_PARAMETER(volume, generic_set_volume);
@@ -194,9 +188,11 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 					    "UTF-8", GenericRecodeFallback, NULL, NULL,
 					    NULL);
     }else{
-	DBG("Warning: Recoding to language default iso, might not be ok...");
-	tmp = module_recode_to_iso(data, bytes, generic_msg_language->name,
-						GenericRecodeFallback);
+	DBG("Warning: Prefered charset not specified, recoding to iso-8859-1");
+	tmp = 
+	    (char*) g_convert_with_fallback(data, bytes, "iso-8859-2",
+					    "UTF-8", GenericRecodeFallback, NULL, NULL,
+					    NULL);
     }
 
     if (tmp == NULL) return -1;
@@ -235,30 +231,25 @@ module_pause(void)
 {
     DBG("pause requested\n");
     if(generic_speaking){
-
         DBG("Sending request to pause to child\n");
         generic_pause_requested = 1;
-        DBG("Waiting in pause for child to terminate\n");
-        while(generic_speaking) usleep(10);
 
         DBG("paused at byte: %d", generic_position);
-        return generic_position;
-        
+        return 0;        
     }else{
-        return 0;
+        return -1;
     }
 }
 
-int
+char*
 module_is_speaking(void)
 {
-    return generic_speaking; 
+    return NULL ; 
 }
 
 void
 module_close(int status)
 {
-    
     DBG("generic: close()\n");
 
     if(generic_speaking){
@@ -267,14 +258,11 @@ module_close(int status)
 
     if (module_terminate_thread(generic_speak_thread) != 0)
         exit(1);
-    
-    CLOSE_DEBUG_FILE();
 
     exit(status);
 }
 
 
-
 /* Internal functions */
 
 char*
@@ -330,6 +318,8 @@ _generic_speak(void* nothing)
             continue;
         }
 
+	module_report_event_begin();
+
         /* Create a new process so that we could send it signals */
         generic_pid = fork();
 
@@ -349,15 +339,18 @@ _generic_speak(void* nothing)
 		/* Set this process as a process group leader (so that SIGKILL
 		   is also delivered to the child processes created by system()) */
 		if (setpgid(0,0) == -1) DBG("Can't set myself as project group leader!");
-		
+
 		e_string = strdup(GenericExecuteSynth);
-		
+
 		e_string = string_replace(e_string, "$PITCH", generic_msg_pitch_str);
-		e_string = string_replace(e_string, "$RATE", generic_msg_rate_str);                       
-		e_string = string_replace(e_string, "$VOLUME", generic_msg_volume_str);                       
+		e_string = string_replace(e_string, "$RATE", generic_msg_rate_str);
+		e_string = string_replace(e_string, "$VOLUME", generic_msg_volume_str);
 		e_string = string_replace(e_string, "$LANGUAGE", generic_msg_language->name);
-		e_string = string_replace(e_string, "$VOICE", generic_msg_voice_str);
-				
+		if (generic_msg_voice_str != NULL)
+		    e_string = string_replace(e_string, "$VOICE", generic_msg_voice_str);
+		else
+		    e_string = string_replace(e_string, "$VOICE", "no_voice");
+
 		/* Cut it into two strings */           
 		p = strstr(e_string, "$DATA");
 		if (p == NULL) exit(1);
@@ -386,7 +379,7 @@ _generic_speak(void* nothing)
             
             generic_speaking = 0;
 
-            module_signal_end();
+	    module_report_event_end();
 
             DBG("child terminated -: status:%d signal?:%d signal number:%d.\n",
                 WIFEXITED(status), WIFSIGNALED(status), WTERMSIG(status));
@@ -448,7 +441,7 @@ _generic_child(TModuleDoublePipe dpipe, const size_t maxlen)
                           strlen(execute_synth_str2) + 8) * sizeof(char));
 
         if (strlen(message->str) != 0){
-            sprintf(command, "%s%s%s", execute_synth_str1, message->str, execute_synth_str2);        
+            sprintf(command, "%s%s%s", execute_synth_str1, message->str, execute_synth_str2);
 
             DBG("child: synth command = |%s|", command);
 
