@@ -18,17 +18,30 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: module_main.c,v 1.5 2004/07/21 08:14:41 hanke Exp $
+ * $Id: module_main.c,v 1.8 2005/10/12 15:59:21 hanke Exp $
  */
 
 /* So that gcc doesn't comply */
 int getline(char**, int*, FILE*);
 
-#define PROCESS_CMD(command, function) if (!strcmp(cmd_buf, #command"\n")){ \
- if (printf("%s\n", (char*) function()) < 0){ \
+#define PROCESS_CMD(command, function) \
+if (!strcmp(cmd_buf, #command"\n")){ \
+ char *msg; \
+ pthread_mutex_lock(&module_stdout_mutex); \
+ if (printf("%s\n", msg = (char*) function()) < 0){ \
      DBG("Broken pipe, exiting...\n"); \
      module_close(2); \
  } \
+ pthread_mutex_unlock(&module_stdout_mutex);\
+ xfree(msg); \
+ fflush(stdout); \
+}
+
+#define PROCESS_CMD_NRP(command, function) \
+if (!strcmp(cmd_buf, #command"\n")){ \
+ pthread_mutex_lock(&module_stdout_mutex); \
+ function(); \
+ pthread_mutex_unlock(&module_stdout_mutex);\
  fflush(stdout); \
 }
 
@@ -40,20 +53,12 @@ main(int argc, char *argv[])
     int ret_init;
     int n;
     char *configfilename;
-    char *status_info;
+    char *status_info;        
 
     module_num_dc_options = 0;
     
     if (argc >= 2){
-        char *tailptr;
-        SPDSemaphore = strtol(argv[1], &tailptr, 0);
-        if (tailptr == argv[1]) SPDSemaphore = -1;        
-    }else{
-        SPDSemaphore = -1;
-    }
-
-    if (argc >= 3){
-        configfilename = strdup(argv[2]);
+        configfilename = strdup(argv[1]);
     }else{
         configfilename = NULL;
     }
@@ -95,10 +100,10 @@ main(int argc, char *argv[])
     if (!strcmp(cmd_buf, "INIT\n")){
 	if (ret_init == 0){
 	    printf("299-%s\n", status_info);
-	    ret = printf("%s\n", "299 OK LOADED SUCCESSFULLY\n");
+	    ret = printf("%s\n", "299 OK LOADED SUCCESSFULLY");
 	}else{
 	    printf("399-%s\n", status_info);
-	    ret = printf("%s\n", "399 ERR CANT INIT MODULE\n");
+	    ret = printf("%s\n", "399 ERR CANT INIT MODULE");
 	    return -1;
 	}
       	xfree(status_info);
@@ -112,6 +117,7 @@ main(int argc, char *argv[])
 	DBG("ERROR: Wrong communication from module client: didn't call INIT\n");
 	module_close(3);
     }
+    xfree(cmd_buf);
 
     while(1){
         cmd_buf = NULL;  n=0;
@@ -121,19 +127,22 @@ main(int argc, char *argv[])
             module_close(2); 
         }
 
+	DBG("CMD: <%s>", cmd_buf);
+
 	PROCESS_CMD(SPEAK, do_speak) 
         else PROCESS_CMD(SOUND_ICON, do_sound_icon)
         else PROCESS_CMD(CHAR, do_char)
         else PROCESS_CMD(KEY, do_key)
-        else PROCESS_CMD(STOP, do_stop) 
-        else PROCESS_CMD(PAUSE, do_pause) 
+        else PROCESS_CMD_NRP(STOP, do_stop) 
+        else PROCESS_CMD_NRP(PAUSE, do_pause) 
         else PROCESS_CMD(SET, do_set) 
-        else PROCESS_CMD(SPEAKING, do_speaking) 
         else PROCESS_CMD(QUIT, do_quit) 
         else{
           printf("300 ERR UNKNOWN COMMAND\n"); 
           fflush(stdout);
         }
+	
+	xfree(cmd_buf);
     } 
 }
 
