@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: module_utils.c,v 1.33 2004/10/18 19:09:15 hanke Exp $
+ * $Id: module_utils.c,v 1.39 2006/01/08 13:36:57 hanke Exp $
  */
 
 #include "fdsetconv.h"
@@ -26,6 +26,7 @@
 
 #include "module_utils.h"
 
+extern char* module_index_mark;
 
 void*
 xmalloc(size_t size)
@@ -78,7 +79,7 @@ do_message(EMessageType msgtype)
         n = 0;
         ret = getline(&cur_line, &n, stdin);
         nlines++;
-        if (ret == -1) return "401 ERROR INTERNAL";
+        if (ret == -1) return strdup("401 ERROR INTERNAL");
 
         if (!strcmp(cur_line, "..\n")){
             xfree(cur_line);
@@ -86,22 +87,24 @@ do_message(EMessageType msgtype)
         }else if (!strcmp(cur_line, ".\n")){
             /* Strip the trailing \n */
             msg->str[strlen(msg->str)-1] = 0;
+	    xfree(cur_line);
             break;
         }
         g_string_append(msg, cur_line);
+	xfree(cur_line);
     }
     
 
     if ((msgtype != MSGTYPE_TEXT) && (nlines > 2)){
-        return "305 DATA MORE THAN ONE LINE";
+        return strdup("305 DATA MORE THAN ONE LINE");
     }
   
     ret = module_speak(msg->str, strlen(msg->str), msgtype);
 
     g_string_free(msg,1);
-    if (ret <= 0) return "301 ERROR CANT SPEAK";
+    if (ret <= 0) return strdup("301 ERROR CANT SPEAK");
     
-    return "200 OK SPEAKING";
+    return strdup("200 OK SPEAKING");
 }
 
 char*
@@ -128,23 +131,25 @@ do_key(void)
     return do_message(MSGTYPE_KEY);
 }
 
-char*
+void
 do_stop(void)
 {
     module_stop();
-    return "201 OK STOPPED";
+    return;
 }
 
-char*
+void
 do_pause(void)
 {
-    size_t pos;
-    static char resp[128];
-    pos = module_pause();
-    
-    snprintf(resp, 127, "202-%d\n202 OK PAUSED", pos);
+    int ret;
+   
+    ret = module_pause(); 
+    if (ret){
+	DBG("WARNING: Can't pause");
+	return;
+    }
 
-    return resp;
+    return;
 }
 
 #define SET_PARAM_NUM(name, cond) \
@@ -210,23 +215,16 @@ do_set(void)
         xfree(line);
     }
 
-    if (err == 0) return "203 OK SETTINGS RECEIVED";
-    if (err == 1) return "302 ERROR BAD SYNTAX";
-    if (err == 2) return "303 ERROR INVALID PARAMETER OR VALUE";
+    if (err == 0) return strdup("203 OK SETTINGS RECEIVED");
+    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
     
-    return "401 ERROR INTERNAL"; /* Can't be reached */
+    return strdup("401 ERROR INTERNAL"); /* Can't be reached */
 }
 #undef SET_PARAM_NUM
 #undef SET_PARAM_STR
 
-char*
-do_speaking(void)
-{
-    if (module_is_speaking()) return "205-1\n205 OK SPEAKING STATUS SENT";
-    else return "205-0\n205 OK SPEAKING STATUS SENT";
-}
-
-/* This has to return int (although it doesn't return) so that we could
+/* This has to return int (although it doesn't return at all) so that we could
  * call it from PROCESS_CMD() macro like the other commands that return
  * something */
 int
@@ -239,8 +237,8 @@ do_quit(void)
 }
 
 int
-module_get_message_part(const char* message, char* part, unsigned int *pos, size_t maxlen,
-                        const char* dividers)
+module_get_message_part(const char* message, char* part, unsigned int *pos,
+			size_t maxlen, const char* dividers)
 {    
     int i, n;
     int num_dividers;
@@ -274,14 +272,12 @@ module_get_message_part(const char* message, char* part, unsigned int *pos, size
                 for(n = 0; n <= num_dividers; n++){
                     if ((part[i] == dividers[n])){                        
                         part[i+1] = 0;                
-                        current_index_mark = -1;
                         (*pos)++;
                         return i+1;            
                     }           
                 }
                 if ((message[*pos] == '\n') && (message[*pos+1] == '\n')){
                     part[i+1] = 0;                
-                    current_index_mark = -1;
                     (*pos)++;
                     return i+1;
                 }
@@ -289,7 +285,6 @@ module_get_message_part(const char* message, char* part, unsigned int *pos, size
                     if(((message[*pos] == '\r') && (message[*pos+1] == '\n'))
                        && ((message[*pos+2] == '\r') && (message[*pos+3] == '\n'))){
                         part[i+1] = 0;                
-                        current_index_mark = -1;
                         (*pos)++;
                         return i+1;
                     }
@@ -309,7 +304,6 @@ module_strip_punctuation_some(char *message, char *punct_chars)
 {
     int len;
     char *p = message;
-    int pchar_bytes;
     int i;
     assert(message != NULL);
 
@@ -436,7 +430,7 @@ module_speak_thread_wfork(sem_t *semaphore, pid_t *process_pid,
             
             *speaking_flag = 0;
 
-            module_signal_end();
+	    module_report_event_end();
 
             DBG("child terminated -: status:%d signal?:%d signal number:%d.\n",
                 WIFEXITED(status), WIFSIGNALED(status), WTERMSIG(status));
@@ -448,7 +442,7 @@ size_t
 module_parent_wfork(TModuleDoublePipe dpipe, const char* message, EMessageType msgtype,
                     const size_t maxlen, const char* dividers, int *pause_requested)
 {
-    int pos = 0;
+    size_t pos = 0;
     char msg[16];
     char *buf;
     int bytes;
@@ -469,11 +463,11 @@ module_parent_wfork(TModuleDoublePipe dpipe, const char* message, EMessageType m
 
         DBG("Returned %d bytes from get_part\n", bytes);
 
-        if (*pause_requested && (current_index_mark!=-1)){               
-            DBG("Pause requested in parent, position %d\n", current_index_mark);                
+        if (*pause_requested){               
+            DBG("Pause requested in parent");                
             module_parent_dp_close(dpipe);
             *pause_requested = 0;
-            return current_index_mark;
+            return 0;
         }             
    
         if (bytes > 0){
@@ -732,12 +726,60 @@ semaphore_post(int sem_id)
     return semop(sem_id, &sem_b, 1);
 }
 
+
 void
-module_signal_end(void)
+module_send_asynchronous(char *text)
 {
-    semaphore_post(SPDSemaphore);
+    pthread_mutex_lock(&module_stdout_mutex);
+    DBG("Printing reply: %s", text);
+    fprintf(stdout, text);
+    fflush(stdout);
+    DBG("Printed");
+    pthread_mutex_unlock(&module_stdout_mutex);
 }
 
+void
+module_report_index_mark(char *mark)
+{
+    char *reply;
+    DBG("Event: Index mark %s", mark);
+    if (mark != NULL)
+	reply = g_strdup_printf("700-%s\n700 INDEX MARK\n", mark);
+    else
+    	return;
+    
+    module_send_asynchronous(reply);
+ 
+    xfree(reply);
+}
+
+void
+module_report_event_begin(void)
+{
+    module_send_asynchronous("701 BEGIN\n");
+}
+
+void
+module_report_event_end(void)
+{
+    module_send_asynchronous("702 END\n");
+}
+
+void
+module_report_event_stop(void)
+{
+    module_send_asynchronous("703 STOP\n");
+}
+
+void
+module_report_event_pause(void)
+{
+    module_send_asynchronous("704 PAUSE\n");
+}
+
+
+
+/* --- CONFIGURATION --- */
 configoption_t *
 module_add_config_option(configoption_t *options, int *num_options, char *name, int type,
                   dotconf_callback_t callback, info_t *info,
@@ -790,3 +832,11 @@ add_config_option(configoption_t *options, int *num_config_options, char *name,
     return opts;
 }
 
+int
+module_utils_init(void)
+{
+    /* Init mutex */
+    pthread_mutex_init(&module_stdout_mutex, NULL);
+
+    return 0;
+}
