@@ -19,12 +19,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: config.c,v 1.6 2004/08/04 08:11:59 hanke Exp $
+ * $Id: config.c,v 1.11 2006/01/08 13:36:57 hanke Exp $
  */
 
 #include <dotconf.h>
 
 #include "speechd.h"
+#include "config.h"
+#include "intl/fdsetconv.h"
 
 static TFDSetClientSpecific *cl_spec_section;
 
@@ -60,7 +62,7 @@ free_config_options(configoption_t *opts, int *num)
 
     if (opts == NULL) return;
 
-    for(i==0; i<=(*num)-1; i++){
+    for(i=0; i<=(*num)-1; i++){
         spd_free((char*) opts[i].name);
     }
     spd_free(opts);
@@ -246,6 +248,7 @@ DOTCONF_CB(cb_AddModule)
     char *module_name;
     char *module_prgname;
     char *module_cfgfile;
+    char *module_dbgfile;
 
     OutputModule *cur_mod;
 
@@ -254,8 +257,9 @@ DOTCONF_CB(cb_AddModule)
 
     module_prgname = cmd->data.list[1];
     module_cfgfile = cmd->data.list[2];
+    module_dbgfile = cmd->data.list[3];
 
-    cur_mod = load_output_module(module_name, module_prgname, module_cfgfile);
+    cur_mod = load_output_module(module_name, module_prgname, module_cfgfile, module_dbgfile);
     if (cur_mod == NULL){
         MSG(3, "Couldn't load specified output module");
         return NULL;
@@ -289,6 +293,7 @@ DOTCONF_CB(cb_BeginClient)
 
     MSG(3, "Reading configuration for pattern %s", cl_spec->pattern);
 
+    /*  Warning: If you modify this, you must also modify update_cl_settings() in set.c !*/
     SET_PAR(rate, -101)
     SET_PAR(pitch, -101)
     SET_PAR(volume, -101)
@@ -297,6 +302,7 @@ DOTCONF_CB(cb_BeginClient)
     SET_PAR(voice, -1)
     SET_PAR(cap_let_recogn, -1)
     SET_PAR(pause_context, -1);
+    SET_PAR(ssml_mode, -1);
     SET_PAR_STR(language)
     SET_PAR_STR(output_module)
     
@@ -311,6 +317,9 @@ DOTCONF_CB(cb_EndClient)
         FATAL("Configuration: Already outside the client specific section!");
     
     client_specific_settings = g_list_append(client_specific_settings, cl_spec_section);
+
+    cl_spec_section = NULL;
+
     return NULL;
 }
 
@@ -385,6 +394,7 @@ load_default_global_set_options()
     GlobalFDSet.min_delay_progress = 2000;
     GlobalFDSet.pause_context = 0;
     GlobalFDSet.ssml_mode = 0;
+    GlobalFDSet.notification = NOTIFY_NOTHING;
 
     SpeechdOptions.max_history_messages = 10000;
 
