@@ -20,31 +20,21 @@
   * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   * Boston, MA 02111-1307, USA.
   *
-  * $Id: index_marking.c,v 1.10 2004/10/12 20:27:22 hanke Exp $
+  * $Id: index_marking.c,v 1.13 2006/01/08 13:36:57 hanke Exp $
   */
 
 #include "index_marking.h"
 
-/* Insert index marks to the given message _msg_. Index marks have
-the form @number@, the numbers begin with 0. It also escape `@' with
-`@@'.*/
-
-#define SD_MARK_HEAD "<mark name=\"sdm_"
-#define SD_MARK_TAIL "\"/>"
-
 void
 insert_index_marks(TSpeechDMessage *msg, int ssml_mode)
 {
     GString *marked_text;
-    int i;
-    size_t len;
     char* pos;
     char character[6];
     char character2[6];
     gunichar u_char;
     int n = 0;
     int ret;
-    int insert = 0;
     int inside_tag = 0;
 
     marked_text = g_string_new("");
@@ -66,7 +56,7 @@ insert_index_marks(TSpeechDMessage *msg, int ssml_mode)
 	    }
 	    else g_string_append_printf(marked_text, "&lt;");
 	}
-        else if (u_char == '>'){
+	else if (u_char == '>'){
             if (ssml_mode){
 		inside_tag = 0;
 		g_string_append_printf(marked_text, "%s", character);
@@ -82,12 +72,12 @@ insert_index_marks(TSpeechDMessage *msg, int ssml_mode)
 	    }
 	}
         else if (((u_char == '.') || (u_char == '?') || (u_char == '!')) && !inside_tag){
-            pos = g_utf8_find_next_char(pos, NULL);   
+            pos = g_utf8_find_next_char(pos, NULL);
             ret = spd_utf8_read_char(pos, character2);
             if ((ret == 0) || (strlen(character2) == 0)){
                 g_string_append_printf(marked_text, "%s", character);
                 MSG2(6, "index_marking", "MSG altering 1: |%s|", marked_text->str);
-                break;            
+                break;
             }
             u_char = g_utf8_get_char(character2);
             if (g_unichar_isspace(u_char)){
@@ -95,8 +85,10 @@ insert_index_marks(TSpeechDMessage *msg, int ssml_mode)
                 n++;
                 MSG2(6, "index_marking", "MSG altering 2: |%s|", marked_text->str);
             }else{
-                g_string_append_printf(marked_text, "%s%s", character, character2);
+                g_string_append_printf(marked_text, "%s", character);
+		//		pos = g_utf8_find_prev_char(pos, NULL);
                 MSG2(6, "index_marking", "MSG altering 3: |%s|", marked_text->str);
+		continue;
             }
         }
         else{
@@ -151,7 +143,10 @@ strip_index_marks(char *buf, int ssml_mode)
     char *p;
     char *p_old;
 
-    str = g_string_new("<speak>");
+    if (ssml_mode)
+	str = g_string_new("<speak>");
+    else
+	str = g_string_new("");
     
     MSG2(5, "index_marking", "Message before stripping index marks: |%s|", buf);
     
@@ -168,7 +163,6 @@ strip_index_marks(char *buf, int ssml_mode)
 	    break;
 	}
 	do{
-	    MSG(5, "p++");
 	    p++;
 	}while(*p != '>' && *p != '\0');
 	if (*p == '>') p++;
