@@ -19,11 +19,16 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: module.c,v 1.21 2004/10/18 19:09:34 hanke Exp $
+ * $Id: module.c,v 1.28 2006/01/08 13:36:57 hanke Exp $
  */
 
+#define _GNU_SOURCE
+
 #include <sys/wait.h>
+#include <sys/stat.h>
+#include <stdio.h>
 #include "speechd.h"
+#include "output.h"
 
 void
 destroy_module(OutputModule *module)
@@ -35,15 +40,13 @@ destroy_module(OutputModule *module)
 }
 
 OutputModule*
-load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile)
+load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile, char* mod_dbgfile)
 {
     OutputModule *module;
     int fr;
     char *arg1;
-    char *arg2;
     int cfg = 0;
     int ret;
-    char buf[256];
 
     if (mod_name == NULL) return NULL;
     
@@ -52,6 +55,8 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile)
     module->name = (char*) spd_strdup(mod_name);
     module->filename = (char*) spd_get_path(mod_prog, MODULEBINDIR);    
     module->configfilename = (char*) spd_get_path(mod_cfgfile, MODULECONFDIR);
+    if (mod_dbgfile != NULL) module->debugfilename = strdup(mod_dbgfile);
+    else module->debugfilename = NULL;
 
     if (!strcmp(mod_name, "testing")){
         module->pipe_in[1] = 1; /* redirect to stdin */
@@ -65,15 +70,29 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile)
         return NULL;
     }     
 
-    arg1 = g_strdup_printf("%d", speaking_sem_id);
     if (mod_cfgfile){
-        arg2 = g_strdup_printf("%s", module->configfilename);
+        arg1 = g_strdup_printf("%s", module->configfilename);
         cfg=1;
     }
 
-    MSG(5,"Initializing output module %s with binary %s and configuration %s",
-        module->name, module->filename, module->configfilename);
-    
+    /* Open the file for child stderr (logging) redirection */
+    if (module->debugfilename != NULL){
+	module->stderr_redirect = open(module->debugfilename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+	if (module->stderr_redirect == -1)
+	    MSG(1,"ERROR: Openning debug file for %s failed: (error=%d) %s", module->name,
+		module->stderr_redirect, strerror(errno));    
+    }else{
+	module->stderr_redirect = -1;
+    }
+
+    MSG(2,"Initializing output module %s with binary %s and configuration %s",
+        module->name, module->filename, module->configfilename);    
+    if (module->stderr_redirect >= 0)
+	MSG(2,"Output module is logging to file %s", module->debugfilename);    
+    else
+	MSG(2,"Output module is logging to standard error output (stderr)");    
+
+
     fr = fork();
     switch(fr){
     case -1: printf("Can't fork, error! Module not loaded."); return NULL;
@@ -85,18 +104,27 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile)
         ret = dup2(module->pipe_out[1], 1);
         close(module->pipe_out[1]);
         close(module->pipe_out[0]);        
-        
+
+	/* Redirrect stderr to the appropriate logfile */
+	if (module->stderr_redirect >= 0 ){
+	    ret = dup2(module->stderr_redirect, 2);
+	}
+
         if (cfg == 0){
-            if (execlp(module->filename, "", arg1, (char *) 0) == -1){                                
+            if (execlp(module->filename, "", (char *) 0) == -1){
                 exit(1);
             }
         }else{
-            if (execlp(module->filename, "", arg1, arg2, (char *) 0) == -1){
+	    //if (execlp("valgrind", "" ,"--trace-children=yes", module->filename, arg1, arg2, (char *) 0) == -1){
+	    if (execlp(module->filename, "", arg1, (char *) 0) == -1){
                 exit(1);
-            }            
+            }
         }
         assert(0);
     default:
+
+	if (cfg) spd_free(arg1);
+
         module->pid = fr;
         close(module->pipe_in[0]);
         close(module->pipe_out[1]);
@@ -114,14 +142,22 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile)
         module->working = 1;
         MSG(2, "Module %s loaded.", module->name);        
 
+	/* Create a stream from the socket */
+	module->stream_out = fdopen(module->pipe_out[0], "r");
+	if (!module->stream_out) FATAL("Can't create a stream for socket, fdopen() failed.");
+	/* Switch to line buffering mode */
+	ret = setvbuf(module->stream_out, NULL, _IONBF, 4096);
+	if (ret) FATAL("Can't set line buffering, setvbuf failed.");
+
 	MSG(4, "Trying to initialize %s.", module->name);
 	if (output_send_data("INIT\n", module, 0) != 0){
 	    MSG(1, "ERROR: Something wrong with %s, can't initialize", module->name);
+	    output_close(module);
 	    return NULL;
 	}else{
 	    char *rep_line = malloc(1024);
 	    FILE *f;
-	    int n = 1024;
+	    size_t n = 1024;
 	    char s;
 	    GString *reply;
 
@@ -174,7 +210,7 @@ unload_output_module(OutputModule *module)
 {
     assert(module != NULL);
 
-    MSG(1,"module name=%s", module->name);
+    MSG(3,"Unloading module name=%s", module->name);
 
     output_close(module);
 
@@ -193,7 +229,6 @@ reload_output_module(OutputModule *old_module)
 
     assert(old_module != NULL); assert(old_module->name != NULL);
 
-    if (old_module->working) output_module_is_speaking(old_module);
     if (old_module->working) return 0;
 
     MSG(3, "Reloading output module %s", old_module->name);    
@@ -203,9 +238,11 @@ reload_output_module(OutputModule *old_module)
     close(old_module->pipe_out[0]);
 
     new_module = load_output_module(old_module->name, old_module->filename,
-                                    old_module->configfilename);
+                                    old_module->configfilename, 
+				    old_module->debugfilename);
     if (new_module == NULL){
-        MSG(3, "Can't load module %s while reloading modules.", old_module->name);
+        MSG(3, "Can't load module %s while reloading modules.", 
+	    old_module->name);
         return -1;
     }
 
