@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: output.c,v 1.16 2004/10/12 20:27:29 hanke Exp $
+ * $Id: output.c,v 1.27 2006/01/08 13:36:57 hanke Exp $
  */
 
 #include "output.h"
@@ -26,9 +26,6 @@
 #include "fdsetconv.c"
 #include "parse.h"
 
-/* TODO: Correct macro */
-#define TEMP_FAILURE_RETRY(expr) (expr)
-
 void
 output_set_speaking_monitor(TSpeechDMessage *msg, OutputModule *output)
 {
@@ -38,7 +35,7 @@ output_set_speaking_monitor(TSpeechDMessage *msg, OutputModule *output)
     speaking_gid = msg->settings.reparted;
 }
 
-static OutputModule*
+OutputModule*
 get_output_module(const TSpeechDMessage *message)
 {
     OutputModule *output = NULL;
@@ -79,17 +76,64 @@ output_unlock(void)
   {  output_unlock(); \
     return (value); }
 
+char*
+output_read_reply(OutputModule *output)
+{
+    GString *rstr;
+    int bytes;
+    char *line = NULL;
+    size_t N = 0;
+    char *reply;
+    
+    rstr = g_string_new("");
+    
+    /* Wait for activity on the socket, when there is some,
+       read all the message line by line */
+    do{
+	bytes = getline(&line, &N, output->stream_out);	
+	if (bytes == -1){
+	    MSG(2, "Error: Broken pipe to module.");
+	    output->working = 0;
+	    speaking_module = NULL;
+	    output_check_module(output);
+	    return NULL; /* Broken pipe */   
+	}
+	g_string_append(rstr, line);
+	/* terminate if we reached the last line (without '-' after numcode) */
+    }while( !((strlen(line) < 4) || (line[3] == ' ')));
+
+    /* The resulting message received from the socket is stored in reply */
+    reply = rstr->str;
+    g_string_free(rstr, FALSE);
+
+    return reply;
+}
+
+char*
+output_read_reply2(OutputModule *output)
+{
+    int bytes;
+    char *reply;
+    
+
+    reply = malloc( 1024 * sizeof(char));
+
+    bytes = read(output->pipe_out[0], reply, 1024);
+    reply[bytes] = 0;
+    MSG2(1, "output_module", "2Read: %d bytes: <%s>", bytes, reply);
+
+    return reply;
+}
+
 int
 output_send_data(char* cmd, OutputModule *output, int wfr)
 {
     int ret;
-    static char response[256];
-    char *tptr;
-    int num_resp;
+    char *response;
 
     if (output == NULL) return -1;
     if (cmd == NULL) return -1;
-
+    
     ret = TEMP_FAILURE_RETRY(write(output->pipe_in[1], cmd, strlen(cmd)));
     fflush(NULL);
     if (ret == -1){
@@ -99,61 +143,35 @@ output_send_data(char* cmd, OutputModule *output, int wfr)
         output_check_module(output);
         return -1;   /* Broken pipe */
     }
-    MSG2(5, "protocol", "Command sent to output module: |%s|", cmd);
+    MSG2(5, "output_module", "Command sent to output module: |%s| (%d)", cmd, wfr);
+    
+    if (wfr){                   /* wait for reply? */	
+	response = output_read_reply(output);
+	if (response == NULL) return -1;
+
+        MSG2(5, "output_module", "Reply from output module: |%s|", response);
 
-    if (wfr){                   /* wait for reply? */
-        ret = TEMP_FAILURE_RETRY(read(output->pipe_out[0], response, 255));
-        if ((ret == -1) || (ret == 0)){
-            MSG(2, "Error: Broken pipe to module.");
-            output->working = 0;
-            speaking_module = NULL;
-            output_check_module(output);
-            return -1; /* Broken pipe */       
-        }
-        MSG2(5, "protocol", "Reply from output module: |%s|", response);
         if (response[0] == '3'){
             MSG(2, "Error: Module reported error in request from speechd (code 3xx).");
+	    spd_free(response);
             return -2; /* User (speechd) side error */
         }
+
         if (response[0] == '4'){
             MSG(2, "Error: Module reported error in itself (code 4xx).");
+	    spd_free(response);
             return -3; /* Module side error */
         }
+
         if (response[0] == '2'){
-            if (ret > 4){
-                if (response[3] == '-'){
-                    num_resp = strtol(&(response[4]), &tptr, 10);
-                    if (tptr == &(response[4])){
-                        MSG(2, "Error: Invalid response from output module.");
-                        output->working = 0; /* The syntax of what we read and what
-                                                we expect to read is now messed up,
-                                                so better not to use the module more. */
-                        speaking_module = NULL;
-                        return -4;
-                    }
-                    /* Check if we got the whole two lines or just one */
-                    /* TODO: Fix this ugly hack with strstr bellow */
-                    if (strstr(response, "\n2") == NULL){
-                        /* Read the rest of the response (the last line) */
-                        ret = TEMP_FAILURE_RETRY(read(output->pipe_out[0], response, 255));
-                        if ((ret == -1) || (ret == 0)){
-                            MSG(2, "Error: Broken pipe to module. Module crashed?");
-                            output->working = 0;
-                            speaking_module = NULL;
-                            output_check_module(output);
-                            return -1; /* Broken pipe */       
-                        }
-                    }
-                    return num_resp;
-                }
-                return 0;
-            }
-            return 0;
+	    return 0;
         }else{                  /* unknown response */
             MSG(3, "Unknown response from output module!");
+	    spd_free(response);
             return -3;
         }
     }       
+    
     return 0;
 }
 
@@ -281,7 +299,7 @@ output_stop()
     else output = speaking_module;
 
     MSG(4, "Module stop!");     
-    SEND_CMD("STOP");
+    SEND_DATA("STOP\n");
 
     OL_RET(0)
 }
@@ -298,39 +316,124 @@ output_pause()
     else output = speaking_module;
 
     MSG(4, "Module pause!");
-    SEND_CMD_GET_VALUE("PAUSE");
+    SEND_DATA("PAUSE\n");
 
-    OL_RET(-1)
+    OL_RET(0)
 }
 
 int
-output_module_is_speaking(OutputModule *output)
+output_module_is_speaking(OutputModule *output, char **index_mark)
 {
-    int err;
+    char *response;
 
     output_lock();
 
-    SEND_CMD_GET_VALUE("SPEAKING");   
+    MSG(5, "output_module_is_speaking()");
+
+    if (output == NULL){
+        MSG(5, "output==NULL in output_module_is_speaking()");
+    	OL_RET(-1);
+    }
+
+    response = output_read_reply(output);
+    if (response == NULL){
+	*index_mark = NULL;
+	OL_RET(-1);
+    }    
+
+    MSG2(5, "output_module", "Reply from output module: |%s|", response);
+
+    if (strlen(response) < 4){
+	MSG2(2, "output_module",
+	     "Error: Wrong communication from output module! Reply less than four bytes.");
+	OL_RET(-1); 
+    }
+
+    if (response[0] == '3'){
+	MSG(2, "Error: Module reported error in request from speechd (code 3xx).");
+	OL_RET(-2); /* User (speechd) side error */
+    }
+    else if (response[0] == '4'){
+	MSG(2, "Error: Module reported error in itself (code 4xx).");
+	OL_RET(-3); /* Module side error */
+    }
+    else if (response[0] == '2'){
+	if (strlen(response) > 4){
+	    if (response[3] == '-'){
+		char *p;
+		p = strchr(response, '\n');
+		*index_mark = (char*) strndup(response+4, p-response-4);
+		MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
+	    }else{
+		MSG2(2, "output_module", "Error: Wrong communication from output module!"
+		    "Reply on SPEAKING not multi-line.");
+		OL_RET(-1); 
+	    }
+	}
+	OL_RET(0);
+    }else if(response[0] == '7'){
+	MSG2(5, "output_module", "Received event:\n %s", response);
+	if (!strncmp(response, "701", 3)) *index_mark = (char*) strdup("__spd_begin");
+	else if (!strncmp(response, "702", 3)) *index_mark = (char*) strdup("__spd_end");
+	else if (!strncmp(response, "703", 3)) *index_mark = (char*) strdup("__spd_stopped");
+	else if (!strncmp(response, "704", 3)) *index_mark = (char*) strdup("__spd_paused");
+	else if (!strncmp(response, "700", 3)){
+	    char *p;
+	    p = strchr(response, '\n');
+	    MSG2(5, "output_module", "response:|%s|\n p:|%s|", response, p);
+	    *index_mark = (char*) strndup(response+4, p-response-4);
+	    MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
+	}else{
+	    MSG2(2, "output_module", "ERROR: Unknown event received from output module");
+	    OL_RET(-5);
+	}
+	OL_RET(0);
+    }else{                  /* unknown response */
+	MSG(3, "Unknown response from output module!");
+	OL_RET(-3);
+    }
 
     OL_RET(-1)
 }
 
 int
-output_is_speaking()
+output_is_speaking(char **index_mark)
 {
     int err;
     OutputModule *output;
 
-    if (speaking_module == NULL) OL_RET(0)
-    else output = speaking_module;
+    output = speaking_module;
 
-    return output_module_is_speaking(output);   
+    err = output_module_is_speaking(output, index_mark);   
+    if (err < 0){
+	*index_mark = NULL;
+    }
+    return err;
+}
+
+/* Wait until the child _pid_ returns with timeout. Calls waitpid() each 100ms
+ until timeout is exceeded. This is not exact and you should not rely on the 
+ exact time waited. */
+int
+waitpid_with_timeout(pid_t pid, int *status_ptr, int options,
+		     size_t timeout)
+{
+    size_t i;
+    int ret;
+    for (i=0; i<= timeout; i+=100){
+	ret = waitpid(pid, status_ptr, options | WNOHANG);
+	if (ret > 0) return ret;
+	if (ret < 0) return ret;
+	usleep(100 * 1000);	/* Sleep 100 ms */
+    }
+    return 0;    
 }
 
 int
 output_close(OutputModule *module)
 {
     int err;
+    int ret;
     OutputModule *output;
     output = module;
 
@@ -341,15 +444,32 @@ output_close(OutputModule *module)
     assert(output->name != NULL);
     MSG(3, "Closing module \"%s\"...", output->name);
     if (output->working){
-        SEND_CMD("STOP");
+        SEND_DATA("STOP\n");
         SEND_CMD("QUIT");
-        usleep(100);            /* So that the module has some time to exit() correctly */
+        usleep(100);
+	/* So that the module has some time to exit() correctly */
     }
-    kill(module->pid, SIGKILL); /* If the module didn't manage to exit */
 
-    waitpid(module->pid, NULL, WNOHANG);
-    MSG(3, "Ok, closed succesfully.");
-   
+    MSG(4, "Waiting for module pid %d", module->pid);
+    ret = waitpid_with_timeout(module->pid, NULL, 0, 1000); 
+    if (ret > 0){
+	MSG(3, "Ok, closed succesfully.");
+    }else if (ret == 0){
+	int ret2;
+	MSG(1, "ERROR: Timed out when waiting for child cancelation");
+	MSG(3, "Killing the module");
+	kill(module->pid, SIGKILL);
+	MSG(3, "Waiting until the child terminates.");
+	ret2 = waitpid_with_timeout(module->pid, NULL, 0, 1000);
+	if (ret2 > 0){
+	    MSG(3, "Module terminated");
+	}else{
+	    MSG(1, "ERROR: Module is not able to terminate, giving up.");
+	}
+    }else{
+	MSG(1, "ERROR: waitpid() failed when waiting for child (module).");
+    }
+    
     OL_RET(0)
 }
 
@@ -404,7 +524,6 @@ escape_dot(char *otext)
     char *seq;
     GString *ntext;
     char *ootext;
-    char *line;
     char *ret = NULL;
     int len;
 
@@ -452,15 +571,15 @@ escape_dot(char *otext)
     }
 
     if (otext == ootext){
+	g_string_free(ntext, 1);
         ret = otext;
     }else{
         g_string_append(ntext, otext);
         free(ootext);
         ret = ntext->str;
+	g_string_free(ntext, 0);
     }
 
-    g_string_free(ntext, 0);
-
     MSG2(6, "escaping", "Altered text: |%s|", ret);
 
     return ret;
