@@ -18,14 +18,18 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: parse.c,v 1.63 2004/10/12 20:27:36 hanke Exp $
+ * $Id: parse.c,v 1.65 2006/01/08 13:36:57 hanke Exp $
  */
 
+#include <ctype.h>
+
 #include "speechd.h"
 
 #include "set.h"
 #include "history.h"
 #include "msg.h"
+#include "server.h"
+#include "sem_functions.h"
 
 /*
   Parse() receives input data and parses them. It can
@@ -42,36 +46,35 @@
     if(!strcmp(command, cmd_name)){ \
         spd_free(command); \
         if ((allowed_in_block == BLOCK_NO) && SpeechdSocket[fd].inside_block) \
-            return ERR_NOT_ALLOWED_INSIDE_BLOCK; \
+            return strdup(ERR_NOT_ALLOWED_INSIDE_BLOCK); \
         return (char*) (parse_function) (buf, bytes, fd); \
     }
 
 #define NOT_ALLOWED_INSIDE_BLOCK() \
     if(SpeechdSocket[fd].inside_block > 0) \
-        return ERR_NOT_ALLOWED_INSIDE_BLOCK;
+        return strdup(ERR_NOT_ALLOWED_INSIDE_BLOCK);
+
+#define ALLOWED_INSIDE_BLOCK() ;
 
 char* 
 parse(const char *buf, const int bytes, const int fd)
 {
     TSpeechDMessage *new;
-    TFDSetElement *settings;
-    GList *gl;
     char *command;
-    char *param;
-    int helper;
-    char *ret;
-    int r, i;
-    int v;
+    int r;
     int end_data;
     char *pos;
     int reparted;
+    int msg_uid;
+    GString* ok_queued_reply;
+    char *reply;
 
     end_data = 0;
 	
     assert(fd > 0);
     if ((buf == NULL) || (bytes == 0)){
         if(SPEECHD_DEBUG) FATAL("invalid buffer for parse()\n");
-        return "ERR INTERNAL";
+        return strdup(ERR_INTERNAL);
     }	
    	
     /* First the condition that we are not in data mode and we
@@ -87,7 +90,7 @@ parse(const char *buf, const int bytes, const int fd)
 
         if (command == NULL){
             if(SPEECHD_DEBUG) FATAL("Invalid buffer for parse()\n");
-            return ERR_INTERNAL; 
+            return strdup(ERR_INTERNAL); 
         }		
 
         CHECK_SSIP_COMMAND("set", parse_set, BLOCK_OK);
@@ -110,24 +113,23 @@ parse(const char *buf, const int bytes, const int fd)
             speechd_connection_destroy(fd);
             /* This is internal Speech Dispatcher message, see serve() */
             spd_free(command);
-            return "999 CLIENT GONE"; /* This is an internal message, not part of SSIP */
+            return strdup("999 CLIENT GONE"); /* This is an internal message, not part of SSIP */
         }
 	
         if (!strcmp(command,"speak")){
+	    spd_free(command);
             /* Ckeck if we have enough space in awaiting_data table for
              * this client, that can have higher file descriptor that
              * everything we got before */
             r = server_data_on(fd);
             if (r!=0){
                 if(SPEECHD_DEBUG) FATAL("Can't switch to data on mode\n");
-                spd_free(command);
-                return ERR_INTERNAL;								 
+                return strdup(ERR_INTERNAL);
             }
-            spd_free(command);
-            return OK_RECEIVE_DATA;
+            return strdup(OK_RECEIVE_DATA);
         }
         spd_free(command);
-        return ERR_INVALID_COMMAND;
+        return strdup(ERR_INVALID_COMMAND);
 
         /* The other case is that we are in awaiting_data mode and
          * we are waiting for text that is comming through the chanel */
@@ -149,7 +151,7 @@ parse(const char *buf, const int bytes, const int fd)
 	    /* TODO: Remove? */
             if ((bytes == 3) && (SpeechdSocket[fd].o_bytes > 2)) SpeechdSocket[fd].o_bytes -= 2;
 
-            if (SpeechdSocket[fd].o_bytes == 0) return OK_MSG_CANCELED;
+            if (SpeechdSocket[fd].o_bytes == 0) return strdup(OK_MSG_CANCELED);
             new = (TSpeechDMessage*) spd_malloc(sizeof(TSpeechDMessage));
             new->bytes = SpeechdSocket[fd].o_bytes;
 	    new->buf = (char*) spd_malloc(new->bytes + 1);
@@ -165,16 +167,20 @@ parse(const char *buf, const int bytes, const int fd)
 	    new->buf = deescape_dot(new->buf);
 
             MSG(5, "New buf is now: |%s|", new->buf);		
-            if(queue_message(new, fd, 1, MSGTYPE_TEXT, reparted) != 0){
+            if((msg_uid = queue_message(new, fd, 1, MSGTYPE_TEXT, reparted)) == 0){
                 if(SPEECHD_DEBUG) FATAL("Can't queue message\n");
                 free(new->buf);
                 free(new);
-                return ERR_INTERNAL;
+                return strdup(ERR_INTERNAL);
             }			       
 
             /* Clear the counter of bytes in the output buffer. */
             server_data_off(fd);
-            return OK_MESSAGE_QUEUED;
+	    ok_queued_reply = g_string_new("");
+	    g_string_printf(ok_queued_reply, C_OK_MESSAGE_QUEUED"-%d\r\n"OK_MESSAGE_QUEUED, msg_uid);
+	    reply = ok_queued_reply->str;
+	    g_string_free(ok_queued_reply, 0);
+            return reply;
         }
 
         {
@@ -201,7 +207,7 @@ parse(const char *buf, const int bytes, const int fd)
     if (end_data == 1) goto enddata;
 
     /* Don't reply on data */
-    return "";
+    return strdup("999 DATA");
 
 }
 #undef CHECK_SSIP_COMMAND
@@ -209,7 +215,7 @@ parse(const char *buf, const int bytes, const int fd)
 #define CHECK_PARAM(param) \
     if (param == NULL){ \
        MSG(3, "Missing parameter from client"); \
-       return ERR_MISSING_PARAMETER; \
+       return strdup(ERR_MISSING_PARAMETER); \
     } 
 
 #define GET_PARAM_INT(name, pos) \
@@ -217,8 +223,9 @@ parse(const char *buf, const int bytes, const int fd)
        char *helper; \
        helper = get_param(buf, pos, bytes, 0); \
        CHECK_PARAM(helper); \
-       if (!isanum(helper)) return ERR_NOT_A_NUMBER; \
+       if (!isanum(helper)) return strdup(ERR_NOT_A_NUMBER); \
        name = atoi(helper); \
+       spd_free(helper); \
    }
 
 #define CONV_DOWN 1
@@ -257,9 +264,9 @@ parse_history(const char *buf, const int bytes, const int fd)
             /* TODO: This needs to be (sim || am)-plified */
             who = get_param(buf,3,bytes, 1);
             CHECK_PARAM(who);
-            if (!strcmp(who, "self")) return ERR_NOT_IMPLEMENTED;
-            if (!strcmp(who, "all")) return ERR_NOT_IMPLEMENTED;                          
-            if (!isanum(who)) return ERR_NOT_A_NUMBER;
+            if (!strcmp(who, "self")) return strdup(ERR_NOT_IMPLEMENTED);
+            if (!strcmp(who, "all")) return strdup(ERR_NOT_IMPLEMENTED);                          
+            if (!isanum(who)) return strdup(ERR_NOT_A_NUMBER);
             
             GET_PARAM_INT(start, 4);
             GET_PARAM_INT(num, 5);
@@ -274,7 +281,7 @@ parse_history(const char *buf, const int bytes, const int fd)
             GET_PARAM_INT(msg_id, 3);
             return (char*) history_get_message(msg_id);
         }else{
-            return ERR_MISSING_PARAMETER;
+            return strdup(ERR_MISSING_PARAMETER);
         }
     }
     else if (TEST_CMD(cmd_main, "cursor")){       
@@ -301,7 +308,7 @@ parse_history(const char *buf, const int bytes, const int fd)
             }
             else{
                 spd_free(location);
-                return ERR_MISSING_PARAMETER;
+                return strdup(ERR_MISSING_PARAMETER);
             }
         }
         else if (TEST_CMD(hist_cur_sub, "forward")){
@@ -314,7 +321,7 @@ parse_history(const char *buf, const int bytes, const int fd)
             return (char*) history_cursor_get(fd);
         }else{
             spd_free(hist_cur_sub);
-            return ERR_MISSING_PARAMETER;
+            return strdup(ERR_MISSING_PARAMETER);
         }
             
     }
@@ -325,15 +332,15 @@ parse_history(const char *buf, const int bytes, const int fd)
     }
     else if (TEST_CMD(cmd_main,"sort")){
         // TODO: everything :)
-        return ERR_NOT_IMPLEMENTED;
+        return strdup(ERR_NOT_IMPLEMENTED);
     }
     else{
         spd_free(cmd_main);
-        return ERR_MISSING_PARAMETER;
+        return strdup(ERR_MISSING_PARAMETER);
     }
 
  
-    return ERR_INVALID_COMMAND;
+    return strdup(ERR_INVALID_COMMAND);
 }
 
 #define SSIP_SET_COMMAND(param) \
@@ -341,6 +348,26 @@ parse_history(const char *buf, const int bytes, const int fd)
         else if (who == 1) ret = set_ ## param ## _uid(uid, param); \
         else if (who == 2) ret = set_ ## param ## _all(param); \
 
+#define SSIP_ON_OFF_PARAM(param, ok_message, err_message, inside_block) \
+    if (!strcmp(set_sub, #param)){ \
+	char *helper_s; \
+	int param; \
+\
+        inside_block \
+\
+        GET_PARAM_STR(helper_s, 3, CONV_DOWN); \
+\
+        if(TEST_CMD(helper_s, "on")) param = 1; \
+        else if(TEST_CMD(helper_s, "off")) param = 0; \
+        else{ \
+            spd_free(helper_s); \
+            return strdup(ERR_PARAMETER_NOT_ON_OFF); \
+        } \
+        SSIP_SET_COMMAND(param); \
+        if (ret) return strdup(err_message); \
+        return strdup(ok_message); \
+    }
+
 char*
 parse_set(const char *buf, const int bytes, const int fd)
 {
@@ -357,10 +384,10 @@ parse_set(const char *buf, const int bytes, const int fd)
     else if (isanum(who_s)){
         who = 1;
         uid = atoi(who_s);
-        spd_free(who_s);
+	spd_free(who_s);
     }else{
         spd_free(who_s);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 
     GET_PARAM_STR(set_sub, 2, CONV_DOWN);
@@ -371,7 +398,7 @@ parse_set(const char *buf, const int bytes, const int fd)
         NOT_ALLOWED_INSIDE_BLOCK();
         
         /* Setting priority only allowed for "self" */
-        if (who != 0) return ERR_COULDNT_SET_PRIORITY; 
+        if (who != 0) return strdup(ERR_COULDNT_SET_PRIORITY); 
         GET_PARAM_STR(priority_s, 3, CONV_DOWN);
 
         if (TEST_CMD(priority_s, "important")) priority = 1;
@@ -381,12 +408,12 @@ parse_set(const char *buf, const int bytes, const int fd)
         else if (TEST_CMD(priority_s, "progress")) priority = 5;
         else{
             spd_free(priority_s);
-            return ERR_UNKNOWN_PRIORITY;
+            return strdup(ERR_UNKNOWN_PRIORITY);
         }
 
         ret = set_priority_self(fd, priority);
-        if (ret) return ERR_COULDNT_SET_PRIORITY;	
-        return OK_PRIORITY_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_PRIORITY);	
+        return strdup(OK_PRIORITY_SET);
     }
     else if (TEST_CMD(set_sub, "language")){
         char *language;
@@ -394,57 +421,58 @@ parse_set(const char *buf, const int bytes, const int fd)
         GET_PARAM_STR(language, 3, CONV_DOWN);
 
         SSIP_SET_COMMAND(language);
-        if (ret) return ERR_COULDNT_SET_LANGUAGE;
-        return OK_LANGUAGE_SET;
+	spd_free(language);
+
+        if (ret) return strdup(ERR_COULDNT_SET_LANGUAGE);
+        return strdup(OK_LANGUAGE_SET);
     }
     else if (TEST_CMD(set_sub, "client_name")){
         char *client_name;
         NOT_ALLOWED_INSIDE_BLOCK();
 
         /* Setting client name only allowed for "self" */
-        if (who != 0) return ERR_COULDNT_SET_CLIENT_NAME;
+        if (who != 0) return strdup(ERR_PARAMETER_INVALID);
       
         GET_PARAM_STR(client_name, 3, CONV_DOWN);
 
         ret = set_client_name_self(fd, client_name);       
         spd_free(client_name);
 
-        if (ret) return ERR_COULDNT_SET_CLIENT_NAME;
-        return OK_CLIENT_NAME_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_CLIENT_NAME);
+        return strdup(OK_CLIENT_NAME_SET);
     }
     else if (!strcmp(set_sub, "rate")){
         signed int rate;
         GET_PARAM_INT(rate, 3);
 
-        if(rate < -100) return ERR_RATE_TOO_LOW;
-        if(rate > +100) return ERR_RATE_TOO_HIGH;
+        if(rate < -100) return strdup(ERR_RATE_TOO_LOW);
+        if(rate > +100) return strdup(ERR_RATE_TOO_HIGH);
 
         SSIP_SET_COMMAND(rate);
-        if (ret) return ERR_COULDNT_SET_RATE;
-        return OK_RATE_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_RATE);
+        return strdup(OK_RATE_SET);
     }
     else if (TEST_CMD(set_sub, "pitch")){
         signed int pitch;
         GET_PARAM_INT(pitch, 3);
 
-        if(pitch < -100) return ERR_PITCH_TOO_LOW;
-        if(pitch > +100) return ERR_PITCH_TOO_HIGH;
+        if(pitch < -100) return strdup(ERR_PITCH_TOO_LOW);
+        if(pitch > +100) return strdup(ERR_PITCH_TOO_HIGH);
 
         SSIP_SET_COMMAND(pitch);
-
-        if (ret) return ERR_COULDNT_SET_PITCH;
-        return OK_PITCH_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_PITCH);
+        return strdup(OK_PITCH_SET);
     }
     else if (TEST_CMD(set_sub, "volume")){
         signed int volume;
         GET_PARAM_INT(volume, 3);
 
-        if(volume < -100) return ERR_VOLUME_TOO_LOW;
-        if(volume > +100) return ERR_VOLUME_TOO_HIGH;
+        if(volume < -100) return strdup(ERR_VOLUME_TOO_LOW);
+        if(volume > +100) return strdup(ERR_VOLUME_TOO_HIGH);
 
         SSIP_SET_COMMAND(volume);
-        if (ret) return ERR_COULDNT_SET_VOLUME;
-        return OK_VOLUME_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_VOLUME);
+        return strdup(OK_VOLUME_SET);
     }
     else if (TEST_CMD(set_sub, "voice")){
         char *voice;
@@ -453,8 +481,8 @@ parse_set(const char *buf, const int bytes, const int fd)
         SSIP_SET_COMMAND(voice);
         spd_free(voice);
 
-        if (ret) return ERR_COULDNT_SET_VOICE;
-        return OK_VOICE_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_VOICE);
+        return strdup(OK_VOICE_SET);
     }
     else if (TEST_CMD(set_sub, "punctuation")){
         char *punct_s;
@@ -468,13 +496,13 @@ parse_set(const char *buf, const int bytes, const int fd)
         else if(TEST_CMD(punct_s,"none")) punctuation_mode = PUNCT_NONE;        
         else{
             spd_free(punct_s);
-            return ERR_PARAMETER_INVALID;
+            return strdup(ERR_PARAMETER_INVALID);
         }
 
         SSIP_SET_COMMAND(punctuation_mode);
 
-        if (ret) return ERR_COULDNT_SET_PUNCT_MODE;
-        return OK_PUNCT_MODE_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_PUNCT_MODE);
+        return strdup(OK_PUNCT_MODE_SET);
     }
     else if (TEST_CMD(set_sub, "output_module")){
         char *output_module;
@@ -484,8 +512,8 @@ parse_set(const char *buf, const int bytes, const int fd)
         SSIP_SET_COMMAND(output_module);
         spd_free(output_module);
 
-        if (ret) return ERR_COULDNT_SET_OUTPUT_MODULE;
-        return OK_OUTPUT_MODULE_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_OUTPUT_MODULE);
+        return strdup(OK_OUTPUT_MODULE_SET);
     }
     else if (TEST_CMD(set_sub, "cap_let_recogn")){
         int capital_letter_recognition;
@@ -498,63 +526,56 @@ parse_set(const char *buf, const int bytes, const int fd)
         else if(TEST_CMD(recognition, "icon")) capital_letter_recognition = RECOGN_ICON;        
         else{
             spd_free(recognition);
-            return ERR_PARAMETER_INVALID;
+            return strdup(ERR_PARAMETER_INVALID);
         }
 
         SSIP_SET_COMMAND(capital_letter_recognition);
 
-        if (ret) return ERR_COULDNT_SET_CAP_LET_RECOG;
-        return OK_CAP_LET_RECOGN_SET;
-    }
-    else if (TEST_CMD(set_sub,"spelling")){
-        char *spelling_s;
-        int spelling;
-        NOT_ALLOWED_INSIDE_BLOCK();
-        GET_PARAM_STR(spelling_s, 3, CONV_DOWN);
-
-        if(TEST_CMD(spelling_s, "on")) spelling = 1;
-        else if(TEST_CMD(spelling_s, "off")) spelling = 0;        
-        else{
-            spd_free(spelling_s);
-            return ERR_PARAMETER_NOT_ON_OFF;
-        }
-
-        SSIP_SET_COMMAND(spelling);
-
-        if (ret) return ERR_COULDNT_SET_SPELLING;
-        return OK_SPELLING_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_CAP_LET_RECOG);
+        return strdup(OK_CAP_LET_RECOGN_SET);
     }
     else if (!strcmp(set_sub, "pause_context")){
         int pause_context;
         GET_PARAM_INT(pause_context, 3);
 
         SSIP_SET_COMMAND(pause_context);
-        if (ret) return ERR_COULDNT_SET_PAUSE_CONTEXT;
-        return OK_PAUSE_CONTEXT_SET;
+        if (ret) return strdup(ERR_COULDNT_SET_PAUSE_CONTEXT);
+        return strdup(OK_PAUSE_CONTEXT_SET);
     }
-    else if (!strcmp(set_sub, "ssml_mode")){
-	char *ssml_mode_s;
-	int ssml_mode;
+    else SSIP_ON_OFF_PARAM(spelling,
+	    OK_SPELLING_SET, ERR_COULDNT_SET_SPELLING,
+	    NOT_ALLOWED_INSIDE_BLOCK())
+    else SSIP_ON_OFF_PARAM(ssml_mode,
+	    OK_SSML_MODE_SET, ERR_COULDNT_SET_SSML_MODE,
+	    ALLOWED_INSIDE_BLOCK())
+    else if (TEST_CMD(set_sub, "notification")){
+	char *scope;
+        char *par_s;
+	int par;
 
-        GET_PARAM_STR(ssml_mode_s, 3, CONV_DOWN);
+	if (who != 0) return strdup(ERR_PARAMETER_INVALID);
 
-        if(TEST_CMD(ssml_mode_s, "on")) ssml_mode = 1;
-        else if(TEST_CMD(ssml_mode_s, "off")) ssml_mode = 0;        
+        GET_PARAM_STR(scope, 3, CONV_DOWN);
+        GET_PARAM_STR(par_s, 4, CONV_DOWN);
+
+        if(TEST_CMD(par_s,"on")) par = 1;
+        else if(TEST_CMD(par_s,"off")) par = 0;        
         else{
-            spd_free(ssml_mode_s);
-            return ERR_PARAMETER_NOT_ON_OFF;
+            spd_free(par_s);
+            return strdup(ERR_PARAMETER_INVALID);
         }
 
-        SSIP_SET_COMMAND(ssml_mode);
-
-        if (ret) return ERR_COULDNT_SET_SSML_MODE;
-        return OK_SSML_MODE_SET;
+	ret = set_notification_self(fd, scope, par);
+	spd_free(scope);
+	           
+        if (ret) return strdup(ERR_COULDNT_SET_NOTIFICATION);
+        return strdup(OK_NOTIFICATION_SET);
     }
     else{
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 
-    return ERR_INVALID_COMMAND;
+    return strdup(ERR_INVALID_COMMAND);
 }
 #undef SSIP_SET_COMMAND
 
@@ -573,21 +594,21 @@ parse_stop(const char *buf, const int bytes, const int fd)
     }
     else if (TEST_CMD(who_s, "self")){
         uid = get_client_uid_by_fd(fd);
-        if(uid == 0) return ERR_INTERNAL;
+        if(uid == 0) return strdup(ERR_INTERNAL);
         speaking_stop(uid);
     }
     else if (isanum(who_s)){
         uid = atoi(who_s);
         spd_free(who_s);
 
-        if (uid <= 0) return ERR_ID_NOT_EXIST;
+        if (uid <= 0) return strdup(ERR_ID_NOT_EXIST);
         speaking_stop(uid);
     }else{
         spd_free(who_s);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 
-    return OK_STOPPED;
+    return strdup(OK_STOPPED);
 }
 
 char*
@@ -605,27 +626,26 @@ parse_cancel(const char *buf, const int bytes, const int fd)
     }
     else if (TEST_CMD(who_s, "self")){
         uid = get_client_uid_by_fd(fd);
-        if(uid == 0) return ERR_INTERNAL;
+        if(uid == 0) return strdup(ERR_INTERNAL);
         speaking_cancel(uid);
     }
     else if (isanum(who_s)){
         uid = atoi(who_s);
         spd_free(who_s);
 
-        if (uid <= 0) return ERR_ID_NOT_EXIST;
+        if (uid <= 0) return strdup(ERR_ID_NOT_EXIST);
         speaking_cancel(uid);
     }else{
         spd_free(who_s);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 
-    return OK_CANCELED;
+    return strdup(OK_CANCELED);
 }
 
 char*
 parse_pause(const char *buf, const int bytes, const int fd)
 {
-    int ret;
     int uid = 0;
     char *who_s;
 
@@ -643,7 +663,7 @@ parse_pause(const char *buf, const int bytes, const int fd)
     }
     else if (TEST_CMD(who_s, "self")){
         uid = get_client_uid_by_fd(fd);
-        if(uid == 0) return ERR_INTERNAL;
+        if(uid == 0) return strdup(ERR_INTERNAL);
         pause_requested = 2;
         pause_requested_fd = fd;
         pause_requested_uid = uid;
@@ -652,23 +672,22 @@ parse_pause(const char *buf, const int bytes, const int fd)
     else if (isanum(who_s)){
         uid = atoi(who_s);
         spd_free(who_s);
-        if (uid <= 0) return ERR_ID_NOT_EXIST;
+        if (uid <= 0) return strdup(ERR_ID_NOT_EXIST);
         pause_requested = 2;
         pause_requested_fd = fd;
         pause_requested_uid = uid;
         speaking_semaphore_post();
     }else{
         spd_free(who_s);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 
-    return OK_PAUSED;
+    return strdup(OK_PAUSED);
 }
 
 char*
 parse_resume(const char *buf, const int bytes, const int fd)
 {
-    int ret;
     int uid = 0;
     char *who_s;
 
@@ -681,44 +700,49 @@ parse_resume(const char *buf, const int bytes, const int fd)
     }
     else if (TEST_CMD(who_s, "self")){
         uid = get_client_uid_by_fd(fd);
-        if(uid == 0) return ERR_INTERNAL;
+        if(uid == 0) return strdup(ERR_INTERNAL);
         speaking_resume(uid);
     }
     else if (isanum(who_s)){
         uid = atoi(who_s);
         spd_free(who_s);
-        if (uid <= 0) return ERR_ID_NOT_EXIST;
+        if (uid <= 0) return strdup(ERR_ID_NOT_EXIST);
         speaking_resume(uid);
     }else{
         spd_free(who_s);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
     
-    return OK_RESUMED;
+    return strdup(OK_RESUMED);
 }
 
 char*
 parse_general_event(const char *buf, const int bytes, const int fd, EMessageType type)
 {
     char *param;
-    int ret;
     TSpeechDMessage *msg;
 
     GET_PARAM_STR(param, 1, NO_CONV);
 
-    if (param == NULL) return ERR_MISSING_PARAMETER;
-    if (param[0] == 0) return ERR_MISSING_PARAMETER;
+    if (param == NULL)	return strdup(ERR_MISSING_PARAMETER);
+    
+    if (param[0] == 0){
+	spd_free(param);
+	return strdup(ERR_MISSING_PARAMETER);
+    }
 
     msg = (TSpeechDMessage*) spd_malloc(sizeof(TSpeechDMessage));
     msg->bytes = strlen(param);
     msg->buf = strdup(param);
 
-    if(queue_message(msg, fd, 1, type, SpeechdSocket[fd].inside_block)){
+    if(queue_message(msg, fd, 1, type, SpeechdSocket[fd].inside_block) == 0){
         if (SPEECHD_DEBUG) FATAL("Couldn't queue message\n");
         MSG(2, "Error: Couldn't queue message!\n");            
     }   
 
-    return OK_MESSAGE_QUEUED;
+    spd_free(param);
+
+    return strdup(OK_MESSAGE_QUEUED);
 }
 
 char*
@@ -762,7 +786,7 @@ parse_list(const char* buf, const int bytes, const int fd)
         return voice_list;
     }else{
         spd_free(list_type);
-        return ERR_PARAMETER_INVALID;
+        return strdup(ERR_PARAMETER_INVALID);
     }
 }
 
@@ -797,21 +821,21 @@ parse_block(const char *buf, const int bytes, const int fd)
         assert(SpeechdSocket[fd].inside_block >= 0);
         if (SpeechdSocket[fd].inside_block == 0){
             SpeechdSocket[fd].inside_block = ++SpeechdStatus.max_gid;
-            return OK_INSIDE_BLOCK;
+            return strdup(OK_INSIDE_BLOCK);
         }else{
-            return ERR_ALREADY_INSIDE_BLOCK;
+            return strdup(ERR_ALREADY_INSIDE_BLOCK);
         }        
     }
     else if (TEST_CMD(cmd_main, "end")){
         assert(SpeechdSocket[fd].inside_block >= 0);
         if (SpeechdSocket[fd].inside_block > 0){
             SpeechdSocket[fd].inside_block = 0;
-            return OK_OUTSIDE_BLOCK;
+            return strdup(OK_OUTSIDE_BLOCK);
         }else{
-            return ERR_ALREADY_OUTSIDE_BLOCK;
+            return strdup(ERR_ALREADY_OUTSIDE_BLOCK);
         }        
     }
-    else return ERR_PARAMETER_INVALID;
+    else return strdup(ERR_PARAMETER_INVALID);
 }
 
 /* TODO: I have no idea how this works. It seems it doesn't
@@ -822,7 +846,6 @@ deescape_dot(char *otext)
     char *seq;
     GString *ntext;
     char *ootext;
-    char *line;
     char *ret = NULL;
     int len;
 
@@ -837,6 +860,7 @@ deescape_dot(char *otext)
     if (strlen(otext) == 2){
         if (!strcmp(otext, "..")){
             otext[1] = 0;
+	    g_string_free(ntext,1);
             return otext;
         }
     }
@@ -873,12 +897,13 @@ deescape_dot(char *otext)
 
     if (otext == ootext){
         ret = otext;
+	g_string_free(ntext, 1);
     }else{
         g_string_append(ntext, otext);
-        free(ootext);
+        spd_free(ootext);
         ret = ntext->str;
+	g_string_free(ntext, 0);
     }
-    g_string_free(ntext, 0);
 
     MSG2(6, "escaping", "Altered text: |%s|", ret);
 
@@ -905,8 +930,8 @@ char*
 get_param(const char *buf, const int n, const int bytes, const int lower_case)
 {
     char* param;
+    char* par;
     int i, y, z;
-    int quote_open = 0;
 
     param = (char*) spd_malloc(bytes);
     assert(param != NULL);
@@ -928,7 +953,7 @@ get_param(const char *buf, const int n, const int bytes, const int lower_case)
     }
 
     if(z <= 0){
-        free(param);
+        spd_free(param);
         return NULL;   
     }
 
@@ -940,10 +965,13 @@ get_param(const char *buf, const int n, const int bytes, const int lower_case)
     }
 
     if(lower_case){
-        param = g_ascii_strdown(param, strlen(param));
-    }
+        par = g_ascii_strdown(param, strlen(param));
+	spd_free(param);
+    }else{
+	par = param;
+    }   
 
-    return param;
+    return par;
 }
 
 /* Read one char  (which _pointer_ is pointing to) from an UTF-8 string
