@@ -18,16 +18,8 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: sem_functions.h,v 1.4 2003/11/06 18:16:41 hanke Exp $
+ * $Id: sem_functions.h,v 1.5 2005/10/16 08:57:59 hanke Exp $
  */
 
-int semaphore_create(key_t key, int flags);
-void semaphore_destroy(int sem_id);
-void semaphore_clear(int sem_id);
-void semaphore_wait(int sem_id);
-void semaphore_post(int sem_id);
-
-int speaking_semaphore_create(key_t key);
-void speaking_semaphore_destroy(void);
 void speaking_semaphore_post(void);
-void speaking_semaphore_wait(void);
+
