@@ -19,9 +19,11 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: set.c,v 1.36 2004/08/04 08:12:07 hanke Exp $
+ * $Id: set.c,v 1.39 2006/01/08 13:36:58 hanke Exp $
  */
 
+#include <fnmatch.h>
+
 #include "set.h"
 #include "alloc.h"
 #include "msg.h"
@@ -233,6 +235,7 @@ set_language_uid(int uid, char *language)
      set->name = strdup(cl_set->val.name); \
             MSG(4,"parameter " #name " set to %s", cl_set->val.name); \
    }
+
 void
 update_cl_settings(gpointer data, gpointer user_data)
 {
@@ -244,6 +247,7 @@ update_cl_settings(gpointer data, gpointer user_data)
 
     if (fnmatch(cl_set->pattern, set->client_name, 0)) return;
 
+    /*  Warning: If you modify this, you must also modify cb_BeginClient in config.c !*/
     CHECK_SET_PAR(rate, -101)
     CHECK_SET_PAR(pitch, -101)
     CHECK_SET_PAR(volume, -101)
@@ -309,6 +313,7 @@ set_output_module_uid(int uid, char* output_module)
 
 SET_SELF_ALL(int, pause_context);
 
+int
 set_pause_context_uid(int uid, int pause_context)
 {
     TFDSetElement *settings;
@@ -322,6 +327,7 @@ set_pause_context_uid(int uid, int pause_context)
 
 SET_SELF_ALL(int, ssml_mode);
 
+int
 set_ssml_mode_uid(int uid, int ssml_mode)
 {
     TFDSetElement *settings;
@@ -333,6 +339,51 @@ set_ssml_mode_uid(int uid, int ssml_mode)
     return 0;
 }
 
+#define SET_NOTIFICATION_STATE(state) \
+	if (val) \
+	    settings->notification = settings->notification | NOTIFY_ ## state; \
+        else \
+	    settings->notification = settings->notification & (! NOTIFY_ ## state); 
+int
+set_notification_self(int fd, char *type, int val)
+{
+    TFDSetElement *settings;
+    int uid;
+
+    uid = get_client_uid_by_fd(fd);
+    if (uid == 0) return 1;
+
+    settings = get_client_settings_by_uid(uid);
+    if (settings == NULL) return 1;
+
+
+    if (!strcmp(type, "begin")){
+	SET_NOTIFICATION_STATE(BEGIN);
+    }else if (!strcmp(type, "end")){
+	SET_NOTIFICATION_STATE(END);
+    }else if (!strcmp(type, "index_marks")){
+	SET_NOTIFICATION_STATE(IM);
+    }else if (!strcmp(type, "pause")){
+	SET_NOTIFICATION_STATE(PAUSE);
+    }else if (!strcmp(type, "resume")){
+	SET_NOTIFICATION_STATE(RESUME);
+    }else if (!strcmp(type, "cancel")){
+	SET_NOTIFICATION_STATE(CANCEL);
+    }else if (!strcmp(type, "all"))
+	{
+	    SET_NOTIFICATION_STATE(END);
+	    SET_NOTIFICATION_STATE(BEGIN);
+	    SET_NOTIFICATION_STATE(IM);
+	    SET_NOTIFICATION_STATE(CANCEL);
+	    SET_NOTIFICATION_STATE(PAUSE);
+	    SET_NOTIFICATION_STATE(RESUME);
+	}
+    else 
+	return -1;
+
+    return 0;
+
+}
 
 
 TFDSetElement*
@@ -355,13 +406,17 @@ default_fd_set(void)
 	new->client_name = spd_strdup(GlobalFDSet.client_name); 
 	new->voice = GlobalFDSet.voice;
 	new->spelling_mode = GlobalFDSet.spelling_mode;         
-	new->cap_let_recogn = GlobalFDSet.cap_let_recogn;       
+	new->cap_let_recogn = GlobalFDSet.cap_let_recogn;      
+
         new->pause_context = GlobalFDSet.pause_context;
 	new->ssml_mode = GlobalFDSet.ssml_mode;
+	new->notification = GlobalFDSet.notification;
+
 	new->active = 1;
 	new->hist_cur_uid = -1;
 	new->hist_cur_pos = -1;
 	new->hist_sorted = 0;
+	new->index_mark = NULL;
 
 	return(new);
 }
