@@ -1,38 +1,45 @@
 
- /*
-  * speaking.c - Speech Dispatcher speech output functions
-  * 
-  * Copyright (C) 2001,2002,2003 Brailcom, o.p.s
-  *
-  * This is free software; you can redistribute it and/or modify it
-  * under the terms of the GNU General Public License as published by
-  * the Free Software Foundation; either version 2, or (at your option)
-  * any later version.
-  *
-  * This software is distributed in the hope that it will be useful,
-  * but WITHOUT ANY WARRANTY; without even the implied warranty of
-  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  * General Public License for more details.
-  *
-  * You should have received a copy of the GNU General Public License
-  * along with this package; see the file COPYING.  If not, write to
-  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-  * Boston, MA 02111-1307, USA.
-  *
-  * $Id: speaking.c,v 1.42 2004/06/28 08:12:25 hanke Exp $
-  */
+/*
+ * speaking.c - Speech Dispatcher speech output functions
+ * 
+ * Copyright (C) 2001,2002,2003 Brailcom, o.p.s
+ *
+ * This is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This software is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this package; see the file COPYING.  If not, write to
+ * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * $Id: speaking.c,v 1.48 2006/01/08 13:36:58 hanke Exp $
+ */
 
 #include <glib.h>
+#include <poll.h>
 #include "speechd.h"
+#include "server.h"
 #include "index_marking.h"
 #include "module.h"
 #include "set.h"
 #include "alloc.h"
-
+#include "msg.h"
+#include "output.h"
 #include "speaking.h"
+#include "sem_functions.h"
 
 TSpeechDMessage *current_message = NULL;
 
+int SPEAKING = 0;
+int poll_count;
+
 /*
   Speak() is responsible for getting right text from right
   queue in right time and saying it loud through the corresponding
@@ -42,17 +49,51 @@ void*
 speak(void* data)
 {
     TSpeechDMessage *message;
-    OutputModule *output;
-    GList *gl = NULL;
-    char *buffer;
     int ret;
-    TSpeechDMessage *msg;
+    struct pollfd *poll_fds; /* Descriptors to poll */
+    struct pollfd main_pfd;
+    struct pollfd helper_pfd;
+    int revents;
 
     /* Block all signals and set thread states */
     set_speak_thread_attributes();
 
+    poll_fds = spd_malloc (2 * sizeof(struct pollfd));
+    
+    main_pfd.fd = speaking_pipe[0];
+    main_pfd.events = POLLIN;
+    main_pfd.revents = 0;
+
+    helper_pfd.fd = -1;
+    helper_pfd.events = POLLIN;
+    helper_pfd.revents = 0;
+    
+
+    poll_fds[0] = main_pfd;
+    poll_fds[1] = helper_pfd;    
+    poll_count = 1;
+
     while(1){
-        speaking_semaphore_wait();
+	ret = poll(poll_fds, poll_count, -1);
+	if(revents = poll_fds[0].revents){
+	    if (revents & POLLIN){
+		char buf[100];
+		MSG(5, "wait_for_poll: activity in Speech Dispatcher");
+		read(poll_fds[0].fd, buf, 1);
+	    }	    
+	}
+	if (poll_count > 1){
+	    if(revents = poll_fds[1].revents){
+		if ((revents & POLLIN) || (revents & POLLPRI)){
+		    MSG(5, "wait_for_poll: activity on output_module");	       
+		    /* Check if sb is speaking or they are all silent. 
+		     * If some synthesizer is speaking, we must wait. */
+		    is_sb_speaking();
+		}        
+	    }	    
+	}
+
+	if (SPEAKING) continue;
 
         /* Handle pause requests */
         if (pause_requested){
@@ -66,33 +107,31 @@ speak(void* data)
             resume_requested = 1;
             continue;
         }
-        
+
         /* Handle resume requests */
         if (resume_requested){
             GList *gl;
-            TSpeechDMessage *element;
+
+	    MSG(5, "Resume requested");
 
             /* Is there any message after resume? */
             if(g_list_length(MessagePausedList) != 0){
                 while(1){
                     pthread_mutex_lock(&element_free_mutex);
                     gl = g_list_find_custom(MessagePausedList, (void*) NULL, message_nto_speak);
+		    MSG(5, "Message insterted back to the queues!");
                     MessagePausedList = g_list_remove_link(MessagePausedList, gl);
                     pthread_mutex_unlock(&element_free_mutex);           
                     if ((gl != NULL) && (gl->data != NULL)){
+			MSG(5, "Reloading message");
                         reload_message((TSpeechDMessage*) gl->data);
                     }else break;                    
                 }
             }
+	    MSG(5, "End of resume processing");
             resume_requested = 0;
         }       
 
-        /* Check if sb is speaking or they are all silent. 
-         * If some synthesizer is speaking, we must wait. */
-        if (is_sb_speaking() == 1){
-            continue;
-        }
-
        pthread_mutex_lock(&element_free_mutex);
         /* Handle postponed priority progress message */
         if ((last_p5_message != NULL) && (g_list_length(MessageQueue->p5) == 0)){
@@ -103,14 +142,15 @@ speak(void* data)
             stop_priority_older_than(2, message->id);
             stop_priority(4);
             stop_priority(5);
+            pthread_mutex_unlock(&element_free_mutex);
             speaking_semaphore_post();
-            pthread_mutex_unlock(&element_free_mutex);         
             continue;
         }else{
             /* Extract the right message from priority queue */
             message = get_message_from_queues();
             if (message == NULL){
-                pthread_mutex_unlock(&element_free_mutex);         
+                pthread_mutex_unlock(&element_free_mutex);
+		MSG(5, "No message in the queue");
                 continue;
             }
         }
@@ -134,21 +174,39 @@ speak(void* data)
         MSG(4,"Message sent to output module");
         if (ret == -1){
             MSG(2, "Error: Output module failed");
-            output_check_module(message);
+            output_check_module(get_output_module(message));
+            pthread_mutex_unlock(&element_free_mutex);
+	    continue;
         }
+	if (ret != 0){
+	    MSG(2, "ERROR: Can't say message. Module reported error in speaking: %d", ret);
+	    pthread_mutex_unlock(&element_free_mutex);
+	    continue;
+	}
+	SPEAKING=1;
+
+	if (speaking_module != NULL){
+	    poll_count = 2;	
+	    helper_pfd.fd = speaking_module->pipe_out[0];
+	    poll_fds[1] = helper_pfd;
+	}
 
         /* Set the id of the client who is speaking. */
         speaking_uid = message->settings.uid;
 
         /* Save the currently spoken message to be able to resume()
-           it after pause */
-        if (current_message != NULL) mem_free_message(current_message);
+           it after pause() */
+        if (current_message != NULL)
+	    if (!current_message->settings.paused_while_speaking) 
+		mem_free_message(current_message);
         current_message = message;
 
         /* Check if the last priority 5 message wasn't said yet */
         if (last_p5_message != NULL){
-            if (last_p5_message->settings.uid == message->settings.uid)
+            if (last_p5_message->settings.uid == message->settings.uid){
+		mem_free_message(last_p5_message);
                 last_p5_message = NULL;
+	    }
         }
 
         pthread_mutex_unlock(&element_free_mutex);        
@@ -162,14 +220,34 @@ reload_message(TSpeechDMessage *msg)
     int im;
     char *pos;
     char *newtext;
+    char *tptr;
 
-    if (msg == NULL) return -1;
-    if (msg->settings.index_mark >= 0){
+    if (msg == NULL){
+	MSG(4, "Warning: msg == NULL in reload_message()");
+	return -1;
+    }
+
+    /* We don't want speak() to free it since we will modify and re-insert it
+       into the queue */
+
+    if (msg->settings.index_mark != NULL){
+	MSG(5, "Recovering index mark %s", msg->settings.index_mark);
         client_settings = get_client_settings_by_uid(msg->settings.uid);
         /* Scroll back to provide context, if required */
-        /* WARNING: This relies on ordered index marks! */
-        im = msg->settings.index_mark + client_settings->pause_context;
-        MSG(5, "index_marking", "Requested index mark is %d (%d+%d)",im,
+        /* WARNING: This relies on ordered SD_MARK_BODY index marks! */
+	MSG(5, "Recovering index mark (number)");
+	im = strtol(msg->settings.index_mark+SD_MARK_BODY_LEN, &tptr, 10);
+	MSG(5, "Recovering index mark (comparing tptr)");
+	if (msg->settings.index_mark+SD_MARK_BODY_LEN == tptr){
+	    MSG2(2, "index_marking", "ERROR: Invalid index_mark '%s'. Message not reloaded.",
+		 msg->settings.index_mark);
+	    return -1;
+	}
+	MSG(5, "Recovered index mark number: %d", im);
+
+        im += client_settings->pause_context;
+
+        MSG2(5, "index_marking", "Requested index mark (with context) is %d (%d+%d)",im,
             msg->settings.index_mark, client_settings->pause_context);
         if (im < 0){
             im = 0;
@@ -186,15 +264,27 @@ reload_message(TSpeechDMessage *msg)
         msg->buf = newtext;
         msg->bytes = strlen(msg->buf);
 
-        if(queue_message(msg, -msg->settings.uid, 0, MSGTYPE_TEXT, 0, 0) != 0){
+        if(queue_message(msg, -msg->settings.uid, 0, MSGTYPE_TEXT, 0) == 0){
             if(SPEECHD_DEBUG) FATAL("Can't queue message\n");
-            spd_free(msg->buf);
-            spd_free(msg);
+	    spd_free(msg->buf);
+	    spd_free(msg);
+            return -1;
+        }
+
+        return 0;
+    }else{
+	MSG(5, "Index mark unknown, inserting the whole message.");
+
+        if(queue_message(msg, -msg->settings.uid, 0, MSGTYPE_TEXT, 0) == 0){
+            if(SPEECHD_DEBUG) FATAL("Can't queue message\n");
+	    spd_free(msg->buf);
+	    spd_free(msg);
             return -1;
         }
 
         return 0;
     }
+    return 0;
 }
 
 void
@@ -294,17 +384,21 @@ speaking_stop_all()
 void
 speaking_cancel(int uid)
 {
+    pthread_mutex_lock(&element_free_mutex);	
     speaking_stop(uid);
     stop_from_uid(uid);
+    pthread_mutex_unlock(&element_free_mutex);
 }
 
 void
 speaking_cancel_all()
 {
     output_stop();
+    pthread_mutex_lock(&element_free_mutex);         
     stop_priority(1);
     stop_priority(2);
     stop_priority(3);
+    pthread_mutex_unlock(&element_free_mutex);         
 }
 
 int
@@ -327,27 +421,40 @@ int
 speaking_pause(int fd, int uid)
 {
     TFDSetElement *settings;
-    TSpeechDMessage *new;
-    char *msg_rest;
-    char *pos;
-    int i;
-    int im;
+    int ret;
+
+    MSG(3, "Pause");
 
     /* Find settings for this particular client */
     settings = get_client_settings_by_uid(uid);
-    if (settings == NULL) return 1;
-    settings->paused = 1;     
+    if (settings == NULL){
+	MSG(3, "ERROR: Can't get settings of active client in speaking_pause()!");
+	return 1;
+    }
+    settings->paused = 1;
 
-    if (is_sb_speaking() == 0) return 0;
-    if (speaking_uid != uid) return 0;    
+    if (speaking_uid != uid){
+	MSG(5, "given uid %d not speaking_uid", speaking_uid, uid);
+	return 0;    
+    }
 
-    im = output_pause();
-    if (im == -1) return 0;
-    if (current_message == NULL) return 0;
+    if (SPEAKING){
+	if (current_message == NULL){
+	    MSG(5, "current_message is null");
+	    return 0;
+	}
+	
+	ret = output_pause();
+	if (ret < 0){
+	    MSG(5, "output_pause returned %d", ret);
+	    return 0;
+	}
     
-    current_message->settings.index_mark = im;
-    MessagePausedList = g_list_append(MessagePausedList, current_message);
-    current_message = NULL;
+	MSG(5, "Including current message into the message paused list");
+	current_message->settings.paused = 2;
+	current_message->settings.paused_while_speaking = 1;
+	MessagePausedList = g_list_append(MessagePausedList, current_message);	
+    }
 
     return 0;  
 }
@@ -372,8 +479,6 @@ int
 speaking_resume(int uid)
 {
     TFDSetElement *settings;
-    TSpeechDMessage *element;
-    GList *gl;
 
     /* Find settings for this particular client */
     settings = get_client_settings_by_uid(uid);
@@ -387,25 +492,152 @@ speaking_resume(int uid)
     return 0;
 }
 
-/* Stops speaking and cancels currently spoken message.*/
 int
-is_sb_speaking()
+socket_send_msg(int fd, char *msg)
 {
-    int speaking = 0;
+    int ret;
 
-    /* Determine is the current module is still speaking */
-    if(speaking_module != NULL){
-        speaking = output_is_speaking();
-    } 
-    if (speaking == 0) speaking_module = NULL;
-    return speaking;
+    assert(msg!=NULL);
+    pthread_mutex_lock(&socket_com_mutex);
+    MSG2(5, "protocol", "%d:REPLY:|%s|", fd, msg);
+    ret = write(fd, msg, strlen(msg));
+    pthread_mutex_unlock(&socket_com_mutex);
+    if (ret < 0){
+	MSG(1, "write() error: %s", strerror(errno));
+	return -1;
+    }
+    return 0;
 }
 
 int
-get_speaking_client_uid()
+report_index_mark(TSpeechDMessage *msg, char *index_mark)
+{
+    char *cmd;
+    int ret;
+
+    cmd = g_strdup_printf(EVENT_INDEX_MARK_C"-%d\r\n"
+			  EVENT_INDEX_MARK_C"-%d\r\n"
+			  EVENT_INDEX_MARK_C"-%s\r\n"
+			  EVENT_INDEX_MARK,
+			  msg->id, msg->settings.uid, index_mark);
+    ret = socket_send_msg(msg->settings.fd, cmd);
+    if (ret){
+	MSG(1, "ERROR: Can't report index mark!");
+	return -1;
+    }
+    spd_free(cmd);
+    return 0;
+}
+
+#define REPORT_STATE(state, ssip_code, ssip_msg) \
+  int \
+  report_ ## state (TSpeechDMessage *msg) \
+  { \
+    char *cmd; \
+    int ret; \
+    cmd = g_strdup_printf(ssip_code"-%d\r\n"ssip_code"-%d\r\n"ssip_msg, \
+	     msg->id, msg->settings.uid); \
+    ret = socket_send_msg(msg->settings.fd, cmd); \
+    if (ret){ \
+      MSG(2, "ERROR: Can't report index mark!"); \
+      return -1; \
+    } \
+    spd_free(cmd); \
+    return 0; \
+  } 
+
+
+REPORT_STATE(begin, EVENT_BEGIN_C, EVENT_BEGIN);
+REPORT_STATE(end, EVENT_END_C, EVENT_END);
+REPORT_STATE(pause, EVENT_PAUSED_C, EVENT_PAUSED);
+REPORT_STATE(resume, EVENT_RESUMED_C, EVENT_RESUMED);
+REPORT_STATE(cancel, EVENT_CANCELED_C, EVENT_CANCELED);
+
+int
+is_sb_speaking(void)
+{
+    int ret;
+    char *index_mark;
+    TFDSetElement *settings;
+
+    MSG(5, "is_sb_speaking(), SPEAKING=%d", SPEAKING);
+
+    /* Determine if the current module is still speaking */
+    if(speaking_module != NULL){
+	if (current_message == NULL){
+	    MSG(1, "Error: Current message is NULL in is_sb_speaking()");
+	    return -1;
+	}
+	settings = &(current_message->settings);	
+
+        ret = output_is_speaking(&index_mark);
+	if (index_mark == NULL) return SPEAKING=0;
+
+	if (!strcmp(index_mark, "no")){
+	    spd_free(index_mark);
+	    return SPEAKING;
+	}
+
+	MSG(5, "INDEX MARK: %s", index_mark);
+
+	if (!strcmp(index_mark, SD_MARK_BODY"begin")){
+	    SPEAKING=1;
+	    if (!settings->paused_while_speaking){
+		if (settings->notification & NOTIFY_BEGIN)
+		    report_begin(current_message);
+	    }else{
+		if (settings->notification & NOTIFY_RESUME)
+		    report_resume(current_message);
+		settings->paused_while_speaking = 0;
+	    }
+	}else if (!strcmp(index_mark, SD_MARK_BODY"end")){
+	    SPEAKING=0;
+	    poll_count=1;
+	    if (settings->notification & NOTIFY_END)
+		report_end(current_message);
+            speaking_semaphore_post();
+	}else if (!strcmp(index_mark, SD_MARK_BODY"paused")){
+	    SPEAKING=0;
+	    poll_count=1;
+	    if (settings->notification & NOTIFY_PAUSE)
+		report_pause(current_message);
+	    /* We don't want to free this message in speak() since we will
+	       later copy it in resume() */
+	    current_message = NULL;
+	}else if (!strcmp(index_mark, SD_MARK_BODY"stopped")){
+	    SPEAKING=0;
+	    poll_count=1;
+	    if (settings->notification & NOTIFY_CANCEL)
+		report_cancel(current_message);
+	    speaking_semaphore_post();
+	}else if (index_mark != NULL){
+	    if (strncmp(index_mark, SD_MARK_BODY, SD_MARK_BODY_LEN)){
+		if (settings->notification & NOTIFY_IM)
+		    report_index_mark(current_message, index_mark);
+	    }else{
+		MSG(5, "Setting current index_mark for the message to %s", index_mark);
+		if (current_message->settings.index_mark != NULL)
+		    free(current_message->settings.index_mark);
+		current_message->settings.index_mark = strdup(index_mark);
+	    }
+	    
+	}
+	spd_free(index_mark);
+    }else{
+	MSG(5, "Speaking module is NULL, SPEAKING==%d", SPEAKING);
+	SPEAKING = 0;
+    }
+
+    if (SPEAKING == 0) speaking_module = NULL;
+
+    return SPEAKING;
+}
+
+int
+get_speaking_client_uid(void)
 {
     int speaking = 0;
-    if(is_sb_speaking() == 0){
+    if(SPEAKING == 0){
         speaking_uid = 0;
         return 0;
     }
@@ -420,12 +652,16 @@ empty_queue(GList *queue)
 {
     int num, i;
     GList *gl;
+    TSpeechDMessage *msg;
 
     num = g_list_length(queue);
     for(i=0;i<=num-1;i++){
         gl = g_list_first(queue);
         assert(gl != NULL);
         assert(gl->data != NULL);
+	msg = (TSpeechDMessage*) gl->data;
+	if (msg->settings.notification & NOTIFY_CANCEL)
+	    report_cancel(msg);
         mem_free_message(gl->data);
         queue = g_list_delete_link(queue, gl);
     }
@@ -439,7 +675,7 @@ empty_queue_by_time(GList *queue, unsigned int uid)
     int num, i;
     GList *gl, *gln;
     TSpeechDMessage *msg;
-
+    
     num = g_list_length(queue);
     gl = g_list_first(queue);
     for(i=0;i<=num-1;i++){
@@ -460,7 +696,6 @@ empty_queue_by_time(GList *queue, unsigned int uid)
 int
 stop_priority(int priority)
 {
-    GList *gl;
     GList *queue;
 
     queue = speaking_get_queue(priority);
@@ -479,7 +714,6 @@ stop_priority(int priority)
 int
 stop_priority_older_than(int priority, unsigned int uid)
 {
-    GList *gl;
     GList *queue;
 
     queue = speaking_get_queue(priority);
@@ -512,15 +746,11 @@ stop_priority_from_uid(GList *queue, const int uid){
 void
 stop_from_uid(const int uid)
 {
-    GList *gl;
-
-    pthread_mutex_lock(&element_free_mutex);
     MessageQueue->p1 = stop_priority_from_uid(MessageQueue->p1, uid);
     MessageQueue->p2 = stop_priority_from_uid(MessageQueue->p2, uid);
     MessageQueue->p3 = stop_priority_from_uid(MessageQueue->p3, uid);
     MessageQueue->p4 = stop_priority_from_uid(MessageQueue->p4, uid);
     MessageQueue->p5 = stop_priority_from_uid(MessageQueue->p5, uid);
-    pthread_mutex_unlock(&element_free_mutex);
 }
 
 /* Determines if this messages is to be spoken
@@ -528,11 +758,10 @@ stop_from_uid(const int uid)
  * Note: If you are wondering why it's reversed (not to speak instead
  * of to speak), it's because we also use this function for
  * searching through the list. */
-static gint
+gint
 message_nto_speak(gconstpointer data, gconstpointer nothing)
 {
     TFDSetElement *global_settings;
-    GList *gl;
     TSpeechDMessage *message = (TSpeechDMessage *) data;
 
     /* Is there something in the body of the message? */
@@ -546,7 +775,7 @@ message_nto_speak(gconstpointer data, gconstpointer nothing)
     else return 1;
 }
 
-static void
+void
 set_speak_thread_attributes()
 {
     int ret;
@@ -617,9 +846,6 @@ stop_priority_except_first(int priority)
 void
 resolve_priorities(int priority)
 {
-    GList *gl;
-    TSpeechDMessage *msg;
-
     if(priority == 1){
         if (highest_priority != 1) output_stop();
         stop_priority(4);
@@ -640,7 +866,7 @@ resolve_priorities(int priority)
 
     if(priority == 4){
         stop_priority_except_first(4);
-        if (is_sb_speaking()){
+        if (SPEAKING){
             if (highest_priority != 4)
                 stop_priority(4);
         }
@@ -648,7 +874,7 @@ resolve_priorities(int priority)
 
     if(priority == 5){
         stop_priority(4);
-        if (is_sb_speaking()){
+        if (SPEAKING){
             GList *gl;
             gl = g_list_last(MessageQueue->p5); 
             MessageQueue->p5 = g_list_remove_link(MessageQueue->p5, gl);
@@ -663,7 +889,7 @@ resolve_priorities(int priority)
 
 }
 
-static TSpeechDMessage*
+TSpeechDMessage*
 get_message_from_queues()
 {
     GList *gl = NULL;
@@ -710,7 +936,7 @@ get_message_from_queues()
     return (TSpeechDMessage *) gl->data;
 }
 
-static GList*
+GList*
 speaking_get_queue(int priority)
 {
     GList *queue = NULL;
@@ -728,7 +954,7 @@ speaking_get_queue(int priority)
     return queue;
 }
 
-static void
+void
 speaking_set_queue(int priority, GList *queue)
 {
     assert(priority > 0  &&  priority <= 5);
