@@ -18,9 +18,11 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: speechd.c,v 1.59 2004/10/12 20:27:49 hanke Exp $
+ * $Id: speechd.c,v 1.64 2006/01/08 13:36:58 hanke Exp $
  */
 
+#include <gmodule.h>
+
 #include "speechd.h"
 
 /* Declare dotconf functions and data structures*/
@@ -32,6 +34,8 @@
 #include "sem_functions.h"
 #include "speaking.h"
 #include "set.h"
+#include "options.h"
+#include "server.h"
 
 /* Manipulating pid files */
 int create_pid_file();
@@ -82,7 +86,7 @@ MSG2(int level, char *kind, char *format, ...)
         va_list args;
         va_list args2;
         int i;
-
+	
         if(std_log) {
             va_start(args, format);
         }
@@ -102,11 +106,14 @@ MSG2(int level, char *kind, char *format, ...)
                 assert(strlen(tstr)>1);
                 tstr[strlen(tstr)-1] = 0;
                 if(std_log) {
-                    fprintf(logfile, "[%s : %d] speechd: ", tstr, tv.tv_usec);
+                    fprintf(logfile, "[%s : %d] speechd: ",
+			    tstr, (int) tv.tv_usec);
                 }
                 if(custom_log) {
-                    fprintf(custom_logfile, "[%s : %d] speechd: ", tstr, tv.tv_usec);
+                    fprintf(custom_logfile, "[%s : %d] speechd: ",
+			    tstr, (int) tv.tv_usec);
                 }
+		spd_free(tstr);
             }
             for(i=1;i<level;i++){
                 if(std_log) {
@@ -160,7 +167,9 @@ MSG(int level, char *format, ...)
                 /* Remove the trailing \n */
                 assert(strlen(tstr)>1);
                 tstr[strlen(tstr)-1] = 0;
-                fprintf(logfile, "[%s : %d] speechd: ", tstr, tv.tv_usec);
+                fprintf(logfile, "[%s : %d] speechd: ",
+			tstr, (int) tv.tv_usec);
+		spd_free(tstr);
             }
             for(i=1;i<level;i++){
                 fprintf(logfile, " ");
@@ -184,9 +193,7 @@ speechd_connection_new(int server_socket)
     struct sockaddr_in client_address;
     unsigned int client_len = sizeof(client_address);
     int client_socket;
-    int v;
     int *p_client_socket, *p_client_uid;
-    GList *gl;
 
     client_socket = accept(server_socket, (struct sockaddr *)&client_address,
                            &client_len);
@@ -242,8 +249,6 @@ int
 speechd_connection_destroy(int fd)
 {
 	TFDSetElement *fdset_element;
-	GList *gl;
-	int v;
 	
 	/* Client has gone away and we remove it from the descriptor set. */
 	MSG(4,"Removing client on fd %d", fd);
@@ -359,12 +364,8 @@ void
 speechd_init(void)
 {
     int START_NUM_FD = 16;
-    configfile_t *configfile = NULL;
-    char *configfilename = SYS_CONF"/speechd.conf" ;
     int ret;
-    char *p;
     int i;
-    int v;
 
 
     SpeechdStatus.max_uid = 0;
@@ -375,6 +376,13 @@ speechd_init(void)
     custom_logfile = NULL;
     custom_log_kind = NULL;
 
+    /* Initialize inter-thread comm pipe */
+    if (pipe(speaking_pipe))
+	{
+	    MSG(1, "Speaking pipe creation failed (%s)", strerror(errno));
+	    FATAL("Can't create pipe");
+	}   
+
     /* Initialize Speech Dispatcher priority queue */
     MessageQueue = (TSpeechDQueue*) speechd_queue_alloc();
     if (MessageQueue == NULL) FATAL("Couldn't alocate memmory for MessageQueue.");
@@ -427,13 +435,8 @@ speechd_init(void)
     ret = pthread_mutex_init(&output_layer_mutex, NULL);
     if(ret != 0) DIE("Mutex initialization failed");
 
-    /* Create a SYS V semaphore */
-    MSG(4, "Creating semaphore");
-    speaking_sem_key = ftok(speechd_pid_file, 1);
-    speaking_sem_id = speaking_semaphore_create(speaking_sem_key);
-    MSG(4, "Testing semaphore");
-    speaking_semaphore_post();
-    speaking_semaphore_wait();
+    ret = pthread_mutex_init(&socket_com_mutex, NULL);
+    if(ret != 0) DIE("Mutex initialization failed");
 
     /* Load configuration from the config file*/
     speechd_load_configuration(0);
@@ -447,6 +450,8 @@ speechd_init(void)
             g_hash_table_size(output_modules) > 1 ? "s" : "" );
     }
 
+
+    last_p5_message = NULL;
     highest_priority = 0;
 }
 
@@ -486,43 +491,40 @@ speechd_load_configuration(int sig)
 void
 speechd_quit(int sig)
 {
-	int ret;
-	
-	MSG(1, "Terminating...");
+    int ret;  
 
-	MSG(2, "Closing open connections...");
-	/* We will browse through all the connections and close them. */
-	g_hash_table_foreach_remove(fd_settings, speechd_client_terminate, NULL);
-	g_hash_table_destroy(fd_settings);
-	
-	MSG(2,"Closing open output modules...");
-	/*  Call the close() function of each registered output module. */
-	g_hash_table_foreach_remove(output_modules, speechd_modules_terminate, NULL);
-	g_hash_table_destroy(output_modules);
-	
-	MSG(2,"Closing server connection...");
-	if(close(server_socket) == -1)
-            MSG(2, "close() failed: %s", strerror(errno));
-	FD_CLR(server_socket, &readfds);
-	
-	MSG(4,"Closing speak() thread...");
-	ret = pthread_cancel(speak_thread);
-	if(ret != 0) FATAL("Speak thread failed to cancel!\n");
-	
-	ret = pthread_join(speak_thread, NULL);
-	if(ret != 0) FATAL("Speak thread failed to join!\n");
+    MSG(1, "Terminating...");    
 
-        MSG(4, "Removing semaphore");
-	speaking_semaphore_destroy();
+    MSG(2, "Closing open connections...");
+    /* We will browse through all the connections and close them. */
+    g_hash_table_foreach_remove(fd_settings, speechd_client_terminate, NULL);
+    g_hash_table_destroy(fd_settings);
 
-        MSG(3, "Removing pid file");
-	destroy_pid_file();
+    MSG(4,"Closing speak() thread...");
+    ret = pthread_cancel(speak_thread);
+    if(ret != 0) FATAL("Speak thread failed to cancel!\n");
+    
+    ret = pthread_join(speak_thread, NULL);
+    if(ret != 0) FATAL("Speak thread failed to join!\n");    
 
-	fflush(NULL);
-
-        MSG(2,"Speech Dispatcher terminated correctly");
-
-	exit(0);	
+    MSG(2,"Closing open output modules...");
+    /*  Call the close() function of each registered output module. */
+    g_hash_table_foreach_remove(output_modules, speechd_modules_terminate, NULL);
+    g_hash_table_destroy(output_modules);
+    
+    MSG(2,"Closing server connection...");
+    if(close(server_socket) == -1)
+	MSG(2, "close() failed: %s", strerror(errno));
+    FD_CLR(server_socket, &readfds);  
+    
+    MSG(3, "Removing pid file");
+    destroy_pid_file();
+    
+    fflush(NULL);
+    
+    MSG(2,"Speech Dispatcher terminated correctly");
+    
+    exit(0);	
 }
 
 
@@ -601,7 +603,6 @@ main(int argc, char *argv[])
 {
     struct sockaddr_in server_address;    
     fd_set testfds;
-    int i, v;
     int fd;
     int ret;
 
@@ -616,32 +617,14 @@ main(int argc, char *argv[])
 
     if (create_pid_file() != 0) exit(1);
 
-    /* Don't print anything on console... */
-    if (spd_mode == SPD_MODE_DAEMON){
-        fclose(stdout);
-        fclose(stderr);
-    }	   
-
     /* Register signals */
     (void) signal(SIGINT, speechd_quit);	
     (void) signal(SIGHUP, speechd_load_configuration);
     (void) signal(SIGPIPE, SIG_IGN);
     (void) signal(SIGUSR1, speechd_reload_dead_modules);
 
-    /* Fork, set uid, chdir, etc. */
-    if (spd_mode == SPD_MODE_DAEMON){
-        daemon(0,0);	   
-        /* Re-create the pid file under this process */
-        unlink(speechd_pid_file);
-        if (create_pid_file() == -1) return -1;
-    }
-
     speechd_init();
 
-    MSG(4,"Creating new thread for speak()");
-    ret = pthread_create(&speak_thread, NULL, speak, NULL);
-    if(ret != 0) FATAL("Speak thread failed!\n");
-
     /* Initialize socket functionality */
     server_socket = socket(AF_INET, SOCK_STREAM, 0);
     {
@@ -667,6 +650,18 @@ main(int argc, char *argv[])
     if (listen(server_socket, 5) == -1)
         FATAL("listen() failed, another Speech Dispatcher running?");
 
+    /* Fork, set uid, chdir, etc. */
+    if (spd_mode == SPD_MODE_DAEMON){
+        daemon(0,0);	   
+        /* Re-create the pid file under this process */
+        unlink(speechd_pid_file);
+        if (create_pid_file() == -1) return -1;
+    }
+
+    MSG(4,"Creating new thread for speak()");
+    ret = pthread_create(&speak_thread, NULL, speak, NULL);
+    if(ret != 0) FATAL("Speak thread failed!\n");
+
     FD_ZERO(&readfds);
     FD_SET(server_socket, &readfds);
     SpeechdStatus.max_fd = server_socket;
