@@ -19,12 +19,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: speechd.h,v 1.51 2004/07/21 08:17:06 hanke Exp $
+ * $Id: speechd.h,v 1.54 2006/01/08 13:36:58 hanke Exp $
  */
 
 #ifndef SPEECHDH
 #define SPEECHDH
 
+#define _GNU_SOURCE
+
 #include <netinet/in.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -120,6 +122,7 @@ struct{
 pthread_t speak_thread;
 pthread_mutex_t element_free_mutex;
 pthread_mutex_t output_layer_mutex;
+pthread_mutex_t socket_com_mutex;
 
 /* Activity requests for the speaking thread are
  handled with SYSV/IPC semaphore */
@@ -159,6 +162,9 @@ TFDSetElement GlobalFDSet;
 /* Variables for socket communication */
 fd_set readfds;
 
+/* Inter thread comm pipe */
+int speaking_pipe[2];
+
 /* Arrays needed for receiving data over socket */
 typedef struct{
     int awaiting_data;
@@ -191,4 +197,20 @@ int isanum(const char *str);
  absolute (starting with slash) or relative. */
 char* spd_get_path(char *filename, char* startdir);
 
+
+/* Functions used in speechd.c only */
+int speechd_connection_new(int server_socket);
+int speechd_connection_destroy(int fd);
+gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user);
+gboolean speechd_modules_terminate(gpointer key, gpointer value, gpointer user);
+void speechd_modules_reload(gpointer key, gpointer value, gpointer user);
+void speechd_reload_dead_modules(int sig);
+void speechd_options_init(void);
+void speechd_init(void);
+void speechd_load_configuration(int sig);
+void speechd_quit(int sig);
+int create_pid_file();
+void destroy_pid_file();
+
+
 #endif
